@@ -641,18 +641,33 @@ public class CodeAttr extends Attribute implements AttrContainer
     return scope;
   }
 
+    /** Create a Scope that is automatically popped.
+     * I.e. the next popScope will keep popping autoPop scopes until
+     * it gets to a non-autoPop scope.  An autoPop Scope is useful for
+     * variables that are assigned and set in the middle of a managed Scope.
+     */
+    public Scope pushAutoPoppableScope() {
+        Scope scope = pushScope();
+        scope.autoPop = true;
+        return scope;
+    }
+
   public Scope getCurrentScope()
   {
     return locals.current_scope;
   }
 
-  public Scope popScope () {
-    Scope scope = locals.current_scope;
-    locals.current_scope = scope.parent;
-    scope.freeLocals(this);
-    scope.end = getLabel();
-    return scope;
-  }
+    public Scope popScope () {
+        Label end = getLabel();
+        for (;;) {
+            Scope scope = locals.current_scope;
+            locals.current_scope = scope.parent;
+            scope.freeLocals(this);
+            scope.end = end;
+            if (! scope.autoPop)
+                return scope;
+        }
+    }
 
   /** Get the index'th parameter. */
   public Variable getArg (int index)
@@ -1183,6 +1198,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   /** Compile code to allocate a new array.
    * The size should have been already pushed on the stack.
    * @param element_type type of the array elements
+   * @params dims number of dimensions - more than 1 is untested
    */
   public void emitNewArray (Type element_type, int dims)
   {
@@ -1206,13 +1222,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 	  }
 	emitNewArray(code);
       }
-    else if (element_type instanceof ObjectType)
-      {
-	reserve(3);
-	put1(189); // anewarray
-	putIndex2(getConstants().addClass((ObjectType) element_type));
-      }
-    else if (element_type instanceof ArrayType)
+    else if (element_type instanceof ArrayType && dims > 1) // untested
     {
       reserve(4);
       put1(197); // multianewarray
@@ -1224,6 +1234,12 @@ public class CodeAttr extends Attribute implements AttrContainer
 	if (popType ().promote () != Type.intType)
 	  throw new Error ("non-int dim. spec. in emitNewArray");
     }
+    else if (element_type instanceof ObjectType)
+      {
+	reserve(3);
+	put1(189); // anewarray
+	putIndex2(getConstants().addClass((ObjectType) element_type));
+      }
     else
       throw new Error ("unimplemented type in emitNewArray");
 
@@ -2464,8 +2480,8 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   /** Compile a tail-call to position 0 of the current procedure.
    * @param pop_args if true, copy argument registers (except this) from stack.
-   * @param scope Scope whose start we jump back to. */
-  public void emitTailCall (boolean pop_args, Scope scope)
+   * @param start the Label to jump back to. */
+  public void emitTailCall (boolean pop_args, Label start)
   {
     if (pop_args)
       {
@@ -2479,7 +2495,15 @@ public class CodeAttr extends Attribute implements AttrContainer
 	    emitStore(locals.used [arg_slots]);
 	  }
       }
-    emitGoto(scope.start);
+    emitGoto(start);
+  }
+
+  /** Compile a tail-call to position 0 of the current procedure.
+   * @param pop_args if true, copy argument registers (except this) from stack.
+   * @param scope Scope whose start we jump back to. */
+  public void emitTailCall (boolean pop_args, Scope scope)
+  {
+    emitTailCall(pop_args, scope.start);
   }
 
   public void processFixups ()
