@@ -440,6 +440,15 @@ public abstract class Type
     return comp == -1 || comp == 0;
   }
 
+    /** If this is the target type, is a given source type compatible?
+     * Return -1 if no; 1 if yes; 0 if need to check at run-time. */
+    public int isCompatibleWithValue(Type valueType) {
+        if (this == toStringType)
+            return 1;
+        int comp = compare(valueType);
+        return comp >= 0 ? 1 : comp == -3 ? -1 : 0;
+    }
+
   /**
    * Computes the common supertype
    *
@@ -458,11 +467,11 @@ public abstract class Type
     if (t2 == neverReturnsType)
       return t1;
     if (t1 == null || t2 == null)
-     return null;
+      return null;
+    if (t1 == t2)
+      return t1;
     if (t1 instanceof PrimType && t2 instanceof PrimType)
       {
-        if (t1 == t2)
-          return t1;
         t1 = ((PrimType) t1).promotedType();
         t2 = ((PrimType) t2).promotedType();
         return t1 == t2 ? t1 : null;
@@ -478,11 +487,15 @@ public abstract class Type
          return Type.objectType;
        ClassType c1 = (ClassType) t1;
        ClassType c2 = (ClassType) t2;
-       if (c1.isInterface() || c2.isInterface())
-         return Type.objectType;
-
-       return lowestCommonSuperType(c1.getSuperclass(), c2.getSuperclass());
+       if (! c1.isInterface() && ! c2.isInterface())
+         {
+           ClassType s1 = c1.getSuperclass();
+           ClassType s2 = c2.getSuperclass();
+           if (s1 != null && s2 != null)
+             return lowestCommonSuperType(s1, s2);
+         }
       }
+    return Type.objectType;
   }
 
   /** Return a numeric code showing "subtype" relationship:
