@@ -4,9 +4,6 @@
 (write x) (newline)
 ; TODO: replace Scheme's (newline) with Common Lisp's (terpri)
 ;; Diagnostic: warnings.lisp:4:8: warning - no declaration seen for x
-; FIXME?: this warning (and the additional one below) occurs when
-; running with RunTestScript and when running "kawa warnings.lisp",
-; but not with "kawa -f warnings.lisp".
 ;; Output: 42
 
 (defun list-of-numbers (start end)
@@ -19,11 +16,11 @@
 (write list-of-numbers) (newline)
 ;; Output: (1 2 3)
 
+(defun unused1 ()
+  (let* ((x y)
+         (x (cdr (list-of-numbers x (+ x 3)))))
+    (write (/ (car (cdr x)) 2)) (newline)))
+;; Diagnostic: warnings.lisp:20:13: warning - no declaration seen for y
+
 (write (apply #'list-of-numbers '(1 5))) (newline)
 ;; Output: (1 2 3 4 5)
-
-(let* ((x x)
-       (x (cdr (list-of-numbers x (+ x 3)))))
-  (write (/ (car (cdr x)) 2)) (newline))
-;; Diagnostic: warnings.lisp:25:11: warning - no declaration seen for x
-;; Output: 22
