@@ -13,6 +13,9 @@ public class ApplicationMainSupport
 
   public static FVector commandLineArguments;
 
+     public static ThreadLocation<String> commandName
+         = new ThreadLocation<String>("command-name");
+
   public static void processSetProperties ()
   {
     String[] args = commandLineArgArray;
@@ -40,26 +43,31 @@ public class ApplicationMainSupport
     setArgs(args, iarg);
   }
 
-  public static void setArgs (String[] args, int arg_start)
-  {
-    int nargs = args.length - arg_start;
-    Object[] array = new Object[nargs];
-    if (arg_start == 0)
-     commandLineArgArray = args;
-    else
-      {
-	String[] strings = new String[nargs];
-	for (int i = nargs;  --i >= 0; )
-	  strings[i] = args[i+arg_start];
-	commandLineArgArray = strings;
-      }
-    for (int i = nargs;  --i >= 0; )
-      array[i] = new FString (args[i + arg_start]);
-    commandLineArguments = new FVector (array);  // FIXME scsh has list
-    // FIXME scsh also has command-line proc
-    Environment.getCurrent().put("command-line-arguments",
-                                 commandLineArguments);
-  }
+    public static void setArgs (String[] args, int arg_start) {
+        if (commandName.get(null) == null) {
+            try {
+                String name = System.getProperty("kawa.command.name");
+                if (name != null)
+                    commandName.set(name);   
+            } catch (Throwable ex) {
+              // Leave commandName unset.
+            }
+        }
+
+        int nargs = args.length - arg_start;
+        if (arg_start == 0)
+            commandLineArgArray = args;
+        else {
+            String[] strings = new String[nargs];
+            for (int i = nargs;  --i >= 0; )
+                strings[i] = args[i+arg_start];
+            commandLineArgArray = strings;
+        }
+    
+        Object[] array = new Object[nargs];
+        System.arraycopy(args, arg_start, array, 0, nargs);
+        commandLineArguments = new ConstVector(array);  // FIXME scsh has list
+    }
 
   public static boolean processSetProperty (String arg)
   {
