@@ -319,8 +319,7 @@ public class ApplyExp extends Expression
         pushArgs(func_lambda, exp.args, exp.args.length, null, comp);
         if (func_lambda.getFlag(LambdaExp.METHODS_COMPILED))
           {
-            popParams(code, func_lambda, null, false);
-            code.emitTailCall(false, func_lambda.getVarScope());
+            code.emitTailCall(false, func_lambda.startForInlining);
             return;
           }
         func_lambda.flags |= LambdaExp.METHODS_COMPILED;
@@ -328,12 +327,15 @@ public class ApplyExp extends Expression
 	comp.curLambda = func_lambda;
 	func_lambda.allocChildClasses(comp);
 	func_lambda.allocParameters(comp);
+        Label startForInlining = new Label(code);
+        startForInlining.define(code);
+        func_lambda.startForInlining = startForInlining;
 	popParams (code, func_lambda, null, false);
 	func_lambda.enterFunction(comp);
 	func_lambda.body.compileWithPosition(comp, target);
 	func_lambda.compileEnd(comp);
 	func_lambda.generateApplyMethods(comp);
-        code.popScope();
+        code.popScope(); // Matches enterScope in allocParameters.
 	comp.curLambda = saveLambda;
 	return;
       }
@@ -418,8 +420,23 @@ public class ApplyExp extends Expression
       }
     if (tail_recurse)
       {
-	popParams(code, func_lambda, incValues, toArray);
-	code.emitTailCall(false, func_lambda.getVarScope());
+        Label startLabel = func_lambda.startForInlining;
+        boolean mustStore = startLabel == null;
+        if (incValues != null && ! mustStore)
+          {
+            for (int i = incValues.length;  --i >= 0; )
+              if (incValues[i] != SetExp.BAD_SHORT)
+                {
+                  mustStore = true;
+                  break;
+                }
+          }
+        if (mustStore)
+          {
+            popParams(code, func_lambda, incValues, toArray);
+            startLabel = func_lambda.getVarScope().getStartLabel();
+          }
+        code.emitTailCall(false, startLabel);
 	return;
       }
     code.emitInvokeVirtual(method);
@@ -442,10 +459,12 @@ public class ApplyExp extends Expression
     return visitor.visitApplyExp(this, d);
   }
 
-  public void visitArgs (InlineCalls visitor)
-  {
-    args = visitor.visitExps(args, args.length, null);
-  }
+    public void visitArgs(InlineCalls visitor) {
+        int nargs = args.length;
+        args = visitor.visitExps(args, nargs,
+                                 isAppendValues() ? null
+                                 : InlineCalls.ValueNeededType.instance);
+    }
 
   protected <R,D> void visitChildren(ExpVisitor<R,D> visitor, D d)
   {
