@@ -58,14 +58,16 @@ public class Compilation implements SourceLocator
   public static final int BODY_PARSED = 4;
   /** State code for lexical bindings having been resolved. */ 
   public static final int RESOLVED = 6;
+  /** State code when initial tree-walking (PushApply) are done. */
+  public static final int PRE_WALKED = 8;
   /** State code when various inlining and optimization passes are done. */
-  public static final int WALKED = 8;
+  public static final int WALKED = 10;
   /** State code that various compile-only data has been determined. */
-  public static final int COMPILE_SETUP = 10;
+  public static final int COMPILE_SETUP = 12;
   /** State code indicating the bytecode has been generated. */
-  public static final int COMPILED = 12;
+  public static final int COMPILED = 14;
   /** State code indicating that bytecode has been written to its target. */
-  public static final int CLASS_WRITTEN = 14;
+  public static final int CLASS_WRITTEN = 16;
   public static final int ERROR_SEEN = 100;
 
   public ModuleInfo minfo;
@@ -115,10 +117,18 @@ public class Compilation implements SourceLocator
     options.add("full-tailcalls",
                 Options.BOOLEAN_OPTION, Boolean.TRUE,
 		"support full tailcalls");
+  public static Options.OptionInfo mainMethodVariable =
+    options.add("main",
+                Options.BOOLEAN_OPTION, Boolean.FALSE,
+                "generate an application, with a main method");
   public static Options.OptionInfo warnUnreachable =
     options.add("warn-unreachable",
                 Options.BOOLEAN_OPTION, Boolean.TRUE,
 		"warn if this code can never be executed");
+  public static Options.OptionInfo warnVoidUsed =
+    options.add("warn-void-used",
+                Options.BOOLEAN_OPTION, Boolean.TRUE,
+		"warn if void-valued expression is used");
   public static Options.OptionInfo warnUndefinedVariable =
     options.add("warn-undefined-variable",
                 Options.BOOLEAN_OPTION, Boolean.TRUE,
@@ -141,6 +151,11 @@ public class Compilation implements SourceLocator
 
   public Options currentOptions = new Options(options);
 
+  public boolean generateMainMethod ()
+  {
+    return currentOptions.getBoolean(mainMethodVariable);
+  }
+  
   public boolean warnUnreachable ()
   {
     return currentOptions.getBoolean(warnUnreachable);
@@ -161,6 +176,10 @@ public class Compilation implements SourceLocator
   {
     return currentOptions.getBoolean(warnUnused);
   }
+  public boolean warnVoidUsed()
+  {
+    return currentOptions.getBoolean(warnVoidUsed);
+  }
   public boolean warnAsError ()
   {
     return currentOptions.getBoolean(warnAsError);
@@ -179,6 +198,9 @@ public class Compilation implements SourceLocator
   }
 
   public static int defaultClassFileVersion =
+    /* #ifdef JAVA8 */
+    // ClassType.JDK_1_8_VERSION
+    /* #else */
     /* #ifdef JAVA7 */
     // ClassType.JDK_1_7_VERSION
     /* #else */
@@ -192,6 +214,7 @@ public class Compilation implements SourceLocator
     /* #endif */
     /* #endif */
     /* #endif */
+    /* #endif */
     ;
 
   /** The default calling convention.
@@ -432,10 +455,6 @@ public class Compilation implements SourceLocator
   /** Rembembers stuff to do in <clinit> of main class. */
   Initializer clinitChain;
 
-  public static boolean generateMainDefault = false;
-  /** True if we should generate a main(String[]) method. */
-  public boolean generateMain = generateMainDefault;
-
   LitTable litTable;
 
   int langOptions;
@@ -470,28 +489,24 @@ public class Compilation implements SourceLocator
     return typeModuleBody;
   }
 
-  /** Emit code to "evaluate" a compile-time constant.
-   * This is the normal external interface.
-   * @param value the value to be compiled
-   */
-  public void compileConstant (Object value)
-  {
-    gnu.bytecode.CodeAttr code = getCode();
-    if (value == null)
-      code.emitPushNull();
-    else if (value instanceof String && ! immediate)
-      code.emitPushString((String) value);
-    else
-      code.emitGetStatic(compileConstantToField(value));
-  }
-
-  public Field compileConstantToField (Object value)
-  {
-    Literal literal = litTable.findLiteral(value);
-    if (literal.field == null)
-      literal.assign(litTable);
-    return literal.field;
-  }
+    /** Emit code to "evaluate" a compile-time constant.
+     * This is the normal external interface.
+     * @param value the value to be compiled
+     */
+    public void compileConstant (Object value) {
+        gnu.bytecode.CodeAttr code = getCode();
+        if (value == null)
+            code.emitPushNull();
+        else if (value instanceof String && ! immediate)
+            code.emitPushString((String) value);
+        else {
+            Literal literal = litTable.findLiteral(value);
+            // if (immediate) maybe we want to use a differnet approach.
+            if (literal.field == null)
+                literal.assign(litTable);
+            code.emitGetStatic(literal.field);
+        }
+    }
 
   public static boolean inlineOk = true;
 
@@ -943,24 +958,6 @@ public class Compilation implements SourceLocator
     this.lexical = lexical;
   }
 
-  /** Shared processing for both compiling/eval. */
-  public void walkModule (ModuleExp mexp)
-  {
-    if (debugPrintExpr)
-      {
-	OutPort dout = OutPort.errDefault();
-	dout.println("[Module:" + mexp.getName());
-	mexp.print(dout);
-	dout.println(']');
-	dout.flush();
-      }
-
-    PushApply.pushApply(mexp, this);
-    InlineCalls.inlineCalls(mexp, this);
-    ChainLambdas.chainLambdas(mexp, this);
-    FindTailCalls.findTailCalls(mexp, this);
-  }
-
   public void outputClass (String directory) throws IOException
   {
     char dirSep = File.separatorChar;
@@ -1189,7 +1186,8 @@ public class Compilation implements SourceLocator
     if (lexp instanceof ClassExp)
       {
 	ClassExp cexp = (ClassExp) lexp;
-	callInitMethods(cexp.getCompiledClassType(this), new Vector<ClassType>(10));
+	callInitMethods(cexp.getCompiledClassType(this),
+                        new ArrayList<ClassType>(10));
       }
 
     code.emitReturn();
@@ -1203,7 +1201,7 @@ public class Compilation implements SourceLocator
    * @param clas Class to search for $finit$, and to search supertypes.
    * @param seen array of seen classes, to avoid duplicate $finit$ calls.
    */
-  void callInitMethods (ClassType clas, Vector<ClassType> seen)
+  void callInitMethods (ClassType clas, ArrayList<ClassType> seen)
   {
     if (clas == null)
       return;
@@ -1213,9 +1211,9 @@ public class Compilation implements SourceLocator
       return;
     // Check for duplicates.
     for (int i = seen.size();  --i >= 0; )
-      if (((ClassType) seen.elementAt(i)).getName() == name)
+      if (seen.get(i).getName() == name)
 	return;
-    seen.addElement(clas);
+    seen.add(clas);
 
     // Recusive call to emit $finit$ of super-types.  However, don't do that
     // for clas.getSuperclass(), because our <init> will automatically call
@@ -1537,8 +1535,9 @@ public class Compilation implements SourceLocator
 		if (ptype != Type.objectType)
                   {
                     SourceLocator saveLoc2 = messages.swapSourceLocator(var);
-                    CheckedTarget.emitCheckedCoerce(this, source,
-                                                    k+1, ptype);
+                    CheckedTarget.emitCheckedCoerce(this, source, k+1,
+                                                    Type.objectType, ptype,
+                                                    null);
                     messages.swapSourceLocator(saveLoc2);
                   }
 		var = var.nextDecl();
@@ -1735,8 +1734,9 @@ public class Compilation implements SourceLocator
 		if (ptype != Type.objectType)
                   {
                     SourceLocator saveLoc2 = messages.swapSourceLocator(var);
-                    CheckedTarget.emitCheckedCoerce(this, source,
-                                                    k+1, ptype, pvar);
+                    CheckedTarget.emitCheckedCoerce(this, source, k+1,
+                                                    Type.objectType, ptype,
+                                                    pvar);
                     messages.swapSourceLocator(saveLoc2);
                   }
 		var = var.nextDecl();
@@ -1879,7 +1879,7 @@ public class Compilation implements SourceLocator
 
     CodeAttr code = method.startCode();
 
-    if (generateMain || generatingApplet() || generatingServlet())
+    if (generateMainMethod() || generatingApplet() || generatingServlet())
       {
 	ClassType languageType
 	  = (ClassType) Type.make(getLanguage().getClass());
@@ -1933,11 +1933,27 @@ public class Compilation implements SourceLocator
             setState(CLASS_WRITTEN);
           }
 
+        if (wantedState >= PRE_WALKED && getState() < PRE_WALKED)
+          {
+            if (debugPrintExpr) {
+                OutPort dout = OutPort.errDefault();
+                dout.println("[Module:" + mexp.getName());
+                mexp.print(dout);
+                dout.println(']');
+                dout.flush();
+            }
+            PushApply.pushApply(mexp, this);  
+            setState(messages.seenErrors() ? ERROR_SEEN : PRE_WALKED);
+          }
+
         if (wantedState >= WALKED && getState() < WALKED)
           {
-            walkModule(mexp);
+            InlineCalls.inlineCalls(mexp, this);
+            ChainLambdas.chainLambdas(mexp, this);
+            FindTailCalls.findTailCalls(mexp, this);
             setState(messages.seenErrors() ? ERROR_SEEN : WALKED);
           }
+
         if (wantedState >= COMPILE_SETUP && getState() < COMPILE_SETUP)
           {
             litTable = new LitTable(this);
@@ -2106,8 +2122,11 @@ public class Compilation implements SourceLocator
 	    code.emitNew(moduleClass);
 	    code.emitDup(moduleClass);
 	    code.emitInvokeSpecial(moduleClass.constructor);
-            // The $instance field needs to be Public so
+            // The $instance field needs to be public so
             // ModuleContext.findInstance can find it.
+            // It needs to be non-final in case moduleClass!=mainClass.
+            // (The latter should probably be fixed by moving this code
+            // to moduleClass's <clinit>.)
 	    moduleInstanceMainField
 	      = moduleClass.addField("$instance", moduleClass,
 				     Access.STATIC|Access.PUBLIC);
@@ -2220,7 +2239,7 @@ public class Compilation implements SourceLocator
 	code.fixupChain(endLiterals, afterLiterals);
       }
 
-    if (generateMain && curClass == mainClass)
+    if (generateMainMethod() && curClass == mainClass)
       {
 	Type[] args = { new ArrayType(javaStringType) };
 	method = curClass.addMethod("main", Access.PUBLIC|Access.STATIC,
