@@ -168,6 +168,8 @@ public class Declaration
 
   public final void setType(Type type)
   {
+    if (type.isVoid())
+      type = Type.objectType;
     this.type = type;
     if (var != null) var.setType(type);
     typeExp = QuoteExp.getInstance(type);
@@ -260,7 +262,11 @@ public class Declaration
                     Compilation comp, Target target)
   {
     if (target instanceof IgnoreTarget)
-      return;
+      {
+        if (access.getFlag(ReferenceExp.ALLOCATE_ON_STACK_LAST))
+          comp.getCode().emitPop(1);
+        return;
+      }
     Declaration owner = access == null ? null : access.contextDecl();
     Expression dvalue = getValueRaw();
     if (isAlias() && dvalue instanceof ReferenceExp)
@@ -352,6 +358,16 @@ public class Declaration
             else
               code.emitGetStatic(field);
           }
+        else if (isClassField())
+          {
+            String getName = ClassExp.slotToMethodName("get", getName());
+            Method getter = ((ClassExp) context).compiledType
+                .getDeclaredMethod(getName, 0);
+            comp.usedClass(getter.getDeclaringClass());
+            comp.usedClass(getter.getReturnType());
+            loadOwningObject(owner, comp);
+            code.emitInvoke(getter);
+          }
         else if (isIndirectBinding() && comp.immediate && getVariable() == null)
           {
             // This is a bit of a kludge.  See comment in ModuleExp.evalModule.
@@ -379,22 +395,9 @@ public class Declaration
         else
           {
             Variable var = getVariable();
-            ClassExp cl;
-            if (context instanceof ClassExp && var == null
-                && ! getFlag(PROCEDURE)
-                && (cl = (ClassExp) context).isMakingClassPair())
-              {
-                String getName = ClassExp.slotToMethodName("get", getName());
-                Method getter = cl.compiledType.getDeclaredMethod(getName, 0);
-                cl.loadHeapFrame(comp);
-                code.emitInvoke(getter);
-              }
-            else
-              {
-                if (var == null)
-                  var = allocateVariable(code);
-                code.emitLoad(var);
-              }
+            if (var == null)
+              var = allocateVariable(code, true);
+            code.emitLoad(var);
           }
         if (isIndirectBinding()
             && (flags & ReferenceExp.DONT_DEREFERENCE) == 0)
@@ -909,6 +912,10 @@ public class Declaration
   }
 
   public final Variable allocateVariable(CodeAttr code)
+  {
+    return allocateVariable(code, false);
+  }
+  public final Variable allocateVariable(CodeAttr code, boolean autoPopScope)
   {
     if (! isSimple() || var == null)
       {
@@ -924,7 +931,9 @@ public class Declaration
 	  {
 	    Type type = isIndirectBinding() ? Compilation.typeLocation
 	      : getType().getImplementationType();
-	    var = context.getVarScope().addVariable(code, type, vname);
+            Scope scope = autoPopScope ? code.pushAutoPoppableScope()
+                : context.getVarScope();
+	    var = scope.addVariable(code, type, vname);
 	  }
       }
     return var;
