@@ -47,6 +47,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     VarValueTracker valueTracker = new VarValueTracker(this);
 
     public Expression visit (Expression exp, Type required) {
+        Expression exp0 = exp;
         if (! exp.getFlag(Expression.VALIDATED)) {
             exp.setFlag(Expression.VALIDATED); // Protect against cycles.
             exp = super.visit(exp, required);
@@ -54,6 +55,15 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         }
         if (required == ProcedureInCallContext.INSTANCE)
           required = null;
+        if (required instanceof ValueNeededType && exp.getType().isVoid()) {
+            if (exp == QuoteExp.voidExp)
+              return QuoteExp.voidObjectExp;
+            if (comp.warnVoidUsed())
+                comp.error('w', "void-valued expression where value is needed",
+                       exp0);
+            // To avoid cascading warnings.
+            return Compilation.makeCoercion(exp, Type.objectType);
+        }
         return checkType(exp, required);
     }
 
@@ -61,10 +71,12 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         Type expType = exp.getType();
         if (expType == Type.toStringType)
             expType = Type.javalangStringType;
-        int cmp = required == null ? 1 : required.compare(expType);
-        boolean incompatible = cmp == -3;
-        if (incompatible
-            || (cmp == -2 && required.isInterface()
+        int cmp = required == null || expType == Type.neverReturnsType
+            || required == Type.neverReturnsType
+            ? 1
+            : required.isCompatibleWithValue(expType);
+        if (cmp < 0
+            || (cmp == 0 && required.isInterface()
                 && (exp instanceof QuoteExp || exp instanceof LambdaExp))) {
             if (exp instanceof LambdaExp
                 && (required instanceof ClassType
@@ -119,6 +131,11 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         return exp;
     }
 
+    private void setCanAccess(LambdaExp exp, Type required) {
+        if (required != ProcedureInCallContext.INSTANCE)
+            exp.setCanRead(true);
+    }
+
   protected Expression visitApplyExp(ApplyExp exp, Type required)
   {
     Expression func = exp.func;
@@ -318,7 +335,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         LambdaExp lval = decl.getLambdaValue();
         if (lval != null)
           {
-            lval.setCanAccess(required != ProcedureInCallContext.INSTANCE);
+            setCanAccess(lval, required);
             valueTracker.checkUninitializedVariables(lval, exp, null);
           }
         Expression dval = decl.getValue();
@@ -396,7 +413,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
 
   protected Expression visitIfExp (IfExp exp, Type required)
   {
-    Expression test = exp.test.visit(this, null);
+    Expression test = exp.test.visit(this, ValueNeededType.instance);
     if (test instanceof ReferenceExp)
       {
         Declaration decl = ((ReferenceExp) test).getBinding();
@@ -415,17 +432,25 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     if (exitValue == null && exp.else_clause != null)
       exp.else_clause = visit(exp.else_clause, required);
     VarValueTracker.forkPop(this);
-    if (test instanceof QuoteExp)
+    // truth: 1 - test is true; 0: test is false; -1 - test is unknown.
+    int truth = ! (test instanceof QuoteExp) ? -1
+        : comp.getLanguage().isTrue(((QuoteExp) test).getValue()) ? 1 : 0;
+    if (exp.else_clause == null && truth <= 0
+        && required instanceof ValueNeededType) {
+        if (comp.warnVoidUsed())
+            comp.error('w', "missing else where value is required", exp);
+        if (truth == 0)
+            return QuoteExp.voidObjectExp;
+    }
+    if (truth >= 0)
+        return exp.select(truth != 0);
+    if (test.getType().isVoid())
       {
-        boolean truth = comp.getLanguage().isTrue(((QuoteExp) test).getValue());
-        return exp.select(truth);
-      }
-    // The neverReturns case is handled in ChainLambdas.
-    if (test.getType().isVoid() && ! test.neverReturns())
-      {
-        boolean truth = comp.getLanguage().isTrue(Values.empty);
-        comp.error('w', "void-valued condition is always "+truth);
-        return new BeginExp(test, exp.select(truth));
+        boolean voidTrue = comp.getLanguage().isTrue(Values.empty);
+
+        if (comp.warnVoidUsed())
+            comp.error('w', "void-valued condition is always "+(truth!=0));
+        return new BeginExp(test, exp.select(voidTrue));
       }
     return exp;
   }
@@ -530,7 +555,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         if (deferableInit(init) && decl.getValueRaw() == init)
           ; // defer
         else
-          init = visit(init, dtype);
+          init = visit(init, ValueNeededType.make(dtype));
 	decl.setInitValue(init);
       }
 
@@ -583,7 +608,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
 
   protected Expression visitLambdaExp (LambdaExp exp, Type required)
   {
-    exp.setCanAccess(required != ProcedureInCallContext.INSTANCE);
+    setCanAccess(exp, required);
     if (exp.getCallConvention() == Compilation.CALL_WITH_UNSPECIFIED)
       exp.setCallConvention(getCompilation());
     Declaration firstDecl = exp.firstDecl();
@@ -594,7 +619,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
       }
     if (exp.getClass() == LambdaExp.class)
       {
-          if (exp.canFinishCondition != LambdaExp.CAN_FINISH
+          if (exp.canFinishCondition != CanFinishMap.CAN_FINISH
               && exp.canFinishCondition != null) {
               exp.setReturnType(Type.neverReturnsType);
           }
@@ -688,13 +713,26 @@ public class InlineCalls extends ExpExpVisitor<Type> {
       return result;
   }
 
-  protected Expression visitTryExp (TryExp exp, Type required)
-  {
-    if (exp.getCatchClauses() == null && exp.getFinallyClause() == null)
-      return visit(exp.try_clause, required);
-    else
-      return super.visitTryExp(exp, required);
-  }
+    protected Expression visitTryExp (TryExp exp, Type required) {
+        if (exp.getCatchClauses() == null && exp.getFinallyClause() == null)
+            return visit(exp.try_clause, required);
+
+        VarValueTracker.forkPush(this);
+        exp.try_clause = exp.try_clause.visit(this, required);
+        for (CatchClause clause = exp.catch_clauses;
+             clause != null; clause = clause.getNext())  {
+            valueTracker.forkNext();
+            clause.visit(this, required); // FIXME update?
+        }
+        // It is possible none of the try_clause or catch_clauses are executed
+        // before the finally_clause is executed, so treat as an empty branch.
+        if (exp.finally_clause != null)
+            valueTracker.forkNext();
+        VarValueTracker.forkPop(this);
+        if (exp.finally_clause != null)
+            exp.finally_clause = exp.finally_clause.visit(this, null);
+        return exp;
+    }
 
   boolean processingAnnotations;
   /** If currently processing an annotation belonging to a declaration.
@@ -754,8 +792,10 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     if (decl != null && decl.getValueRaw() == exp.new_value
         && deferableInit(exp.new_value))
       ; // defer
-    else 
-      exp.new_value = visit(exp.new_value, decl == null || decl.isAlias() ? null : decl.type);
+    else {
+        Type dtype = decl == null || decl.isAlias() ? null : decl.type;
+        exp.new_value = visit(exp.new_value, ValueNeededType.make(dtype));
+    }
     if (! exp.isDefining() && decl != null && decl.isClassMethod())
       comp.error('e', "can't assign to method "+decl.getName(), exp);
     if (decl != null && decl.getFlag(Declaration.TYPE_SPECIFIED))
@@ -882,12 +922,12 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     if (lexp.keywords != null)
       return null;
     boolean varArgs = lexp.max_args < 0;
-    if ((lexp.min_args == lexp.max_args
-         && lexp.min_args == args.length)
-        || (varArgs && lexp.min_args == 0))
+    int fixed = lexp.min_args;
+    if ((fixed == lexp.max_args
+         && fixed == args.length)
+        || (varArgs && args.length >= fixed))
       {
         Declaration prev = null;
-        int i = 0;
         IdentityHashTable mapper;
         Expression[] cargs;
         if (makeCopy)
@@ -904,11 +944,20 @@ public class InlineCalls extends ExpExpVisitor<Type> {
           }
         if (varArgs)
           {
-            Expression[] xargs = new Expression[args.length+1];
-            xargs[0] = QuoteExp.getInstance(lexp.firstDecl().type);
-            System.arraycopy(args, 0, xargs, 1, args.length);
-            cargs = new Expression[] { new ApplyExp(Invoke.make, xargs) };
+            cargs = new Expression[fixed+1];
+            // Copy over fixed arguments.
+            System.arraycopy(args, 0, cargs, 0, fixed);
+            // Create list/array constructor for rest args.
+            Expression[] xargs = new Expression[args.length-fixed+1];
+            Declaration restArg = lexp.firstDecl();
+            for (int i = fixed;  --i >= 0; )
+                restArg = restArg.nextDecl();
+            xargs[0] = QuoteExp.getInstance(restArg.type);
+            // Copy over rest args.
+            System.arraycopy(args, fixed, xargs, 1, args.length-fixed);
+            cargs[fixed] = new ApplyExp(Invoke.make, xargs);
           }
+        int i = 0;
         LetExp let = new LetExp();
         for (Declaration param = lexp.firstDecl(); param != null; i++)
           {
@@ -931,14 +980,11 @@ public class InlineCalls extends ExpExpVisitor<Type> {
                 lexp.remove(prev, param);
                 let.add(prev, param);
               }
-            if (! varArgs)
-              {
-                if ( ! param.getCanWrite()) {
-                  param.nvalues = 0;
-                  param.values = null;
-                  param.noteValueFromLet(let);
-                }
-              }
+            if ( ! param.getCanWrite()) {
+                param.nvalues = 0;
+                param.values = null;
+                param.noteValueFromLet(let);
+            }
             prev = param;
             param = next;
           }
@@ -988,4 +1034,38 @@ public class InlineCalls extends ExpExpVisitor<Type> {
             return getImplementationType().compare(other.getImplementationType());
         }
     }
+
+    /** A marker type to indicate that void is invalid.
+     * Only used as the required type, in e.g. rhs of assignment.
+     */
+    public static class ValueNeededType extends ObjectType {
+        static final ValueNeededType instance
+            = new ValueNeededType(null);
+
+        Type actualType;
+
+        ValueNeededType(Type actualType) {
+            super("value-needed-type:"+actualType);
+            this.actualType = actualType;
+        }
+
+        public static Type make(Type type) {
+            if (type == null)
+                return instance;
+            if (type instanceof ValueNeededType || type == Type.objectType)
+                return type;
+            /* FUTURE not support by code yet
+            return new ValueNeededType(type);
+            */
+            return type;
+        }
+
+        public Type getImplementationType() {
+            return actualType;
+        }
+
+        public int compare(Type other) {
+            return other.isVoid() ? -1 : 1;
+        }
+    }
 }
