@@ -16,7 +16,7 @@ import gnu.kawa.functions.Convert;
 public class LambdaExp extends ScopeExp
 {
   public Expression body;
-  /** Minimum number of parameters.
+  /** Minimum number of actual arguments.
    * Does not count implicit isThisParameter(). */
   public int min_args;
   /** Maximum number of actual arguments;  -1 if variable. */
@@ -45,6 +45,9 @@ public class LambdaExp extends ScopeExp
    */
   ReferenceExp siblingReferences;
 
+    /** The location for function start, before arguments are stored. */
+    Label startForInlining;
+
   public void capture (Declaration decl)
   {
     if (decl.isSimple())
@@ -134,7 +137,7 @@ public class LambdaExp extends ScopeExp
   Variable closureEnv;
 
   static final int CAN_READ = 2;
-  static final int CAN_CALL = 4;
+  static final int INLINE_ONLY = 4;
   static final int IMPORTS_LEX_VARS = 8;
   static final int NEEDS_STATIC_LINK = 16;
   /* Used (future) by FindTailCalls. */
@@ -147,9 +150,8 @@ public class LambdaExp extends ScopeExp
   public static final int SEQUENCE_RESULT = 1024;
   public static final int OVERLOADABLE_FIELD = 2048;
   public static final int ATTEMPT_INLINE = 4096;
-  static final int INLINE_ONLY = 8192;
-  public static final int IN_EXPWALKER = 0x4000;
-  protected static final int NEXT_AVAIL_FLAG = 0x8000;
+  public static final int IN_EXPWALKER = 0x2000;
+  protected static final int NEXT_AVAIL_FLAG = 0x4000;
 
   /** True iff this lambda is only "called" inline. */
   public final boolean getInlineOnly() { return (flags & INLINE_ONLY) != 0; }
@@ -202,7 +204,7 @@ public class LambdaExp extends ScopeExp
 
   void setCallersNeedStaticLink()
   {
-    LambdaExp outer = outerLambda();
+    LambdaExp outer = nameDecl.getContext().currentLambda();
     for (ApplyExp app = nameDecl.firstCall;  app != null;  app = app.nextCall)
       {
         LambdaExp caller = app.context;
@@ -219,18 +221,6 @@ public class LambdaExp extends ScopeExp
     else flags &= ~CAN_READ;
   }
 
-  public final boolean getCanCall()
-  { return (flags & CAN_CALL) != 0; }
-  public final void setCanCall(boolean called)
-  {
-    if (called) flags |= CAN_CALL;
-    else flags &= ~CAN_CALL;
-  }
-
-    public final void setCanAccess(boolean canRead) {
-        flags |= canRead ? CAN_READ : CAN_CALL;
-    }
-
   /** True if this is a method in an ClassExp. */
   public final boolean isClassMethod()
   { return (flags & CLASS_METHOD) != 0; }
@@ -285,18 +275,14 @@ public class LambdaExp extends ScopeExp
   { return getCallConvention() >= Compilation.CALL_WITH_CONSUMER; }
 
     /** This function can finish if specified functions can finish.
-     * I.e. call this function can complete normally is there is some i
-     * such that all members of canFinishDependencies.get(i) can finish.
+     * I.e. calling this function can complete normally is there is a bit i
+     * such that for all LambdaExp l the mask canFinishCondition.get(l)
+     * bit a zero value for bit i.
      * May be null if there is no dependency yet in the current execution
-     * path fork, in which case PsuhApply.canGinishDeps will realize it.
+     * path fork, in which case PushApply.canFinishDeps will realize it.
      * This value is calculated during PushApply and used in InlineCalls.
      */
-    ArrayList<HashSet<LambdaExp>> canFinishCondition;
-
-    static ArrayList<HashSet<LambdaExp>> CAN_FINISH = new ArrayList();
-    static { CAN_FINISH.add(new HashSet<LambdaExp>()); }
-
-    static ArrayList<HashSet<LambdaExp>> CANNOT_FINISH = new ArrayList();
+    CanFinishMap canFinishCondition;
 
     /** Set of functions whose canFinishCondition may depend on this. */
     Set<LambdaExp> canFinishListeners;
@@ -312,22 +298,13 @@ public class LambdaExp extends ScopeExp
     }
 
     void checkCanFinish() {
-        ArrayList<HashSet<LambdaExp>> cond = canFinishCondition;
+        CanFinishMap cond = canFinishCondition;
         if (cond != null && ! getFlag(LambdaExp.IN_EXPWALKER)) {
             // See if we can simplify exp.canFinishCondition.
             // I.e. if any dependencies are now CAN_FINISH.
-            for (Set<LambdaExp> set : cond) {
-                Iterator<LambdaExp> callees = set.iterator();
-                while (callees.hasNext()) {
-                    LambdaExp callee = callees.next();
-                    if (callee.canFinishCondition == LambdaExp.CAN_FINISH)
-                        callees.remove();
-                }
-                if (set.isEmpty()) {
-                    canFinishCondition = LambdaExp.CAN_FINISH;
-                    notifyCanFinish();
-                    return;
-                }
+            if (cond.canFinish()) {
+                canFinishCondition = CanFinishMap.CAN_FINISH;
+                notifyCanFinish();
             }
         }
     }
@@ -638,6 +615,9 @@ public class LambdaExp extends ScopeExp
 	  {
 	    child.compileAsMethod(comp);
 	  }
+        else if (child instanceof ClassExp) {
+            ((ClassExp) child).compileMembers(comp);
+        }
 	child = child.nextSibling;
       }
 
@@ -1227,7 +1207,7 @@ public class LambdaExp extends ScopeExp
         if (decl == firstArgsArrayArg && argsArray != null)
           {
             getVarScope().addVariable(argsArray);
-          } 
+          }
         if (! getInlineOnly()
             && getCallConvention() >= Compilation.CALL_WITH_CONSUMER
             && (firstArgsArrayArg == null ? decl == null
@@ -1467,8 +1447,7 @@ public class LambdaExp extends ScopeExp
 		defaultStart = 0;
 	      }
 	  }
-        if (param.ignorable())
-          continue;
+        boolean ignorable = param.ignorable();
 	if (plainArgs >= 0 || ! param.isSimple()
 	    || param.isIndirectBinding())
 	  {
@@ -1479,102 +1458,131 @@ public class LambdaExp extends ScopeExp
 	    // slot in the heapFrame.  Thus we emit an aaload instruction.
 	    // Unfortunately, it expects the new value *last*,
 	    // so first push the heapFrame array and the array index.
-	    if (!param.isSimple ())
+	    if (!param.isSimple () && ! ignorable)
 	      param.loadOwningObject(null, comp);
 	    // This part of the code pushes the incoming argument.
 	    if (plainArgs < 0)
 	      {
 		// Simple case:  Use Incoming register.
-		code.emitLoad(param.getVariable());
+                if (! ignorable)
+		  code.emitLoad(param.getVariable());
 	      }
             else if (i < min_args)
 	      { // This is a required parameter, in argsArray[i].
-		code.emitLoad(argsArray);
-		code.emitPushInt(i);
-		code.emitArrayLoad(Type.objectType);
+                if (! ignorable)
+                  {
+                    code.emitLoad(argsArray);
+                    code.emitPushInt(i);
+                    code.emitArrayLoad(Type.objectType);
+                  }
 	      }
             else if (i < min_args + opt_args)
 	      { // An optional parameter
-		code.emitPushInt(i - plainArgs);
-                code.emitLoad(argsArray);
-		code.emitArrayLength();
-		code.emitIfIntLt();
-                code.emitLoad(argsArray);
-		code.emitPushInt(i - plainArgs);
-		code.emitArrayLoad();
-		code.emitElse();
-		param.getInitValue().compile(comp, paramType);
-		code.emitFi();
+                Expression defaultArg = param.getInitValue();
+                if (! ignorable || ! (defaultArg instanceof QuoteExp))
+                  {
+                    code.emitPushInt(i - plainArgs);
+                    code.emitLoad(argsArray);
+                    code.emitArrayLength();
+                    code.emitIfIntLt();
+                    if (! ignorable)
+                      {
+                        code.emitLoad(argsArray);
+                        code.emitPushInt(i - plainArgs);
+                        code.emitArrayLoad();
+                      }
+                    code.emitElse();
+                    if (ignorable)
+                      defaultArg.compile(comp, Target.Ignore);
+                    else
+                      defaultArg.compile(comp, paramType);
+                    code.emitFi();
+                  }
 	      }
 	    else if (max_args < 0 && i == min_args + opt_args)
 	      {
 		// This is the "rest" parameter (i.e. following a "."):
 		// Convert argsArray[i .. ] to a list.
-		code.emitLoad(argsArray);
-		code.emitPushInt(i - plainArgs);
-		code.emitInvokeStatic(Compilation.makeListMethod);
+                if (! ignorable)
+                  {
+                    code.emitLoad(argsArray);
+                    code.emitPushInt(i - plainArgs);
+                    code.emitInvokeStatic(Compilation.makeListMethod);
+                  }
 		stackType = Compilation.scmListType;
               }
 	    else
 	      { // Keyword argument.
-		code.emitLoad(argsArray);
-		code.emitPushInt(min_args + opt_args - plainArgs);
-		comp.compileConstant(keywords[key_i++]);
+                Keyword keyword = keywords[key_i++];
 		Expression defaultArg = param.getInitValue();
-                Type boxedParamType = paramType instanceof PrimType
-                    ? ((PrimType) paramType).boxedType()
-                    : paramType;
+                if (! ignorable || ! (defaultArg instanceof QuoteExp))
+                {
+                    code.emitLoad(argsArray);
+                    code.emitPushInt(min_args + opt_args - plainArgs);
+                    comp.compileConstant(keyword);
+                    Type boxedParamType = paramType instanceof PrimType
+                        ? ((PrimType) paramType).boxedType()
+                        : paramType;
 
-		// We can generate better code if the defaultArg expression
-		// has no side effects.  For simplicity and safety, we just
-		// special case literals, which handles most cases.
-		if (defaultArg instanceof QuoteExp)
-		  {
-		    if (searchForKeywordMethod4 == null)
-		      {
-			Type[] argts = new Type[4];
-			argts[0] = Compilation.objArrayType;
-			argts[1] = Type.intType;
-			argts[2] = Type.objectType;
-			argts[3] = Type.objectType;
-			searchForKeywordMethod4
-			  = Compilation.scmKeywordType.addMethod
-			  ("searchForKeyword",  argts,
-			   Type.objectType, Access.PUBLIC|Access.STATIC);
-		      }
-		    defaultArg.compile(comp, boxedParamType);
-		    code.emitInvokeStatic(searchForKeywordMethod4);
-		  }
-		else
-		  {
-		    if (searchForKeywordMethod3 == null)
-		      {
-			Type[] argts = new Type[3];
-			argts[0] = Compilation.objArrayType;
-			argts[1] = Type.intType;
-			argts[2] = Type.objectType;
-			searchForKeywordMethod3
-			  = Compilation.scmKeywordType.addMethod
-			  ("searchForKeyword",  argts,
-			   Type.objectType, Access.PUBLIC|Access.STATIC);
-		      }
-		    code.emitInvokeStatic(searchForKeywordMethod3);
-		    code.emitDup(1);
-		    comp.compileConstant(Special.dfault);
-		    code.emitIfEq();
-		    code.emitPop(1);
-		    defaultArg.compile(comp, boxedParamType);
-                    paramType.emitCoerceToObject(code);
-		    code.emitFi();
-		  }
+                    // We can generate better code if the defaultArg expression
+                    // has no side effects.  For simplicity and safety, we just
+                    // special case literals, which handles most cases.
+                    if (defaultArg instanceof QuoteExp)
+                      {
+                        if (searchForKeywordMethod4 == null)
+                        {
+                            Type[] argts = new Type[4];
+                            argts[0] = Compilation.objArrayType;
+                            argts[1] = Type.intType;
+                            argts[2] = Type.objectType;
+                            argts[3] = Type.objectType;
+                            searchForKeywordMethod4
+                                = Compilation.scmKeywordType.addMethod
+                                ("searchForKeyword",  argts,
+                                 Type.objectType, Access.PUBLIC|Access.STATIC);
+                        }
+                        defaultArg.compile(comp, boxedParamType);
+                        code.emitInvokeStatic(searchForKeywordMethod4);
+                      }
+                    else
+                      {
+                        if (searchForKeywordMethod3 == null)
+                          {
+                            Type[] argts = new Type[3];
+                            argts[0] = Compilation.objArrayType;
+                            argts[1] = Type.intType;
+                            argts[2] = Type.objectType;
+                            searchForKeywordMethod3
+                                = Compilation.scmKeywordType.addMethod
+                                ("searchForKeyword",  argts,
+                                 Type.objectType, Access.PUBLIC|Access.STATIC);
+                          }
+                        code.emitInvokeStatic(searchForKeywordMethod3);
+                        if (! ignorable)
+                          code.emitDup(1);
+                        comp.compileConstant(Special.dfault);
+                        code.emitIfEq();
+                        if (ignorable)
+                          defaultArg.compile(comp, Target.Ignore);
+                        else
+                          {
+                            code.emitPop(1);
+                            defaultArg.compile(comp, boxedParamType);
+                            paramType.emitCoerceToObject(code);
+                          }
+                        code.emitFi();
+                      }
+                }
 	      }
+            if (ignorable)
+              continue;
 	    // Now finish copying the incoming argument into its
 	    // home location.
             if (paramType != stackType)
-	      CheckedTarget.emitCheckedCoerce(comp, this, i+1, paramType);
+              CheckedTarget.emitCheckedCoerce(comp, this, i+1, stackType, paramType, null);
 	    if (param.isIndirectBinding())
               param.pushIndirectBinding(comp);
-	    if (param.isSimple())
+            if (param.isSimple())
               {
                 Variable var = param.getVariable();
                 if (param.isIndirectBinding())
@@ -2082,24 +2090,26 @@ public class LambdaExp extends ScopeExp
     return bodyFirst;
   }
 
-  /** Check if argument is a this(...) or super(...) initializtion.
-   * If so, return return the corresponding this or super class.
-   */
-  public ClassType checkForInitCall (Expression bodyFirst)
-  {
-    ClassType calledInit = null;
-    Object value;  Expression exp;
-    if (bodyFirst instanceof ApplyExp
-        && (exp = ((ApplyExp) bodyFirst).func) instanceof QuoteExp
-        && (value = ((QuoteExp) exp).getValue()) instanceof PrimProcedure)
-      {
-        PrimProcedure pproc = (PrimProcedure) value;
-        if (pproc.isSpecial()
-            && ("<init>".equals(pproc.method.getName())))
-          calledInit = pproc.method.getDeclaringClass();
-      }
-    return calledInit;
-  }
+    /** Check if argument is a this(...) or super(...) initializtion.
+     * If so, return return the corresponding this or super class.
+     */
+    public ClassType checkForInitCall (Expression bodyFirst) {
+        ClassType calledInit = null;
+        if (bodyFirst instanceof ApplyExp) {
+            Expression exp = ((ApplyExp) bodyFirst).func;
+            if (exp instanceof QuoteExp) {
+                Object value = ((QuoteExp) exp).getValue();
+                if (value instanceof PrimProcedure) {
+                    PrimProcedure pproc = (PrimProcedure) value;
+                    Method meth = pproc.getMethod();
+                    if (pproc.isSpecial()
+                        && ("<init>".equals(meth.getName())))
+                        calledInit = meth.getDeclaringClass();
+                }
+            }
+        }
+        return calledInit;
+    }
 }
 
 class Closure extends MethodProc
