@@ -7,7 +7,7 @@ import gnu.kawa.reflect.ClassMemberLocation;
  * Class for the dummy top-level function of a module.
  */
 
-public abstract class ModuleBody extends Procedure0
+public abstract class ModuleBody extends Procedure0 implements RunnableModule
 {
   public void apply (CallContext ctx)  throws Throwable
   {
@@ -109,7 +109,7 @@ public abstract class ModuleBody extends Procedure0
    * Thus if you start up AWT, the JVM will wait for the AWT to finish,
    * even if there are no other non-daemon threads.
    * So call exitIncrement() each time a Freme is created,
-   * and call exitDecrement() a Frame is closed. */
+   * and call exitDecrement() when a Frame is closed. */
   public static synchronized void exitDecrement()
   {
     int counter = exitCounter;
@@ -128,7 +128,13 @@ public abstract class ModuleBody extends Procedure0
   /** This is invoked by main when ModuleBody is compiled with --main. */
   public final void runAsMain ()
   {
-    gnu.text.WriterManager.instance.registerShutdownHook();
+    runAsMain(this);
+  }
+
+  /** This is invoked by main when ModuleBody is compiled with --main. */
+  public static void runAsMain (RunnableModule module)
+  {
+    boolean registered = gnu.text.WriterManager.instance.registerShutdownHook();
     try
       {
 	CallContext ctx = CallContext.getInstance();
@@ -136,17 +142,18 @@ public abstract class ModuleBody extends Procedure0
 	  {
 	    OutPort out = OutPort.outDefault();
 	    ctx.consumer = kawa.Shell.getOutputConsumer(out);
-	    run(ctx);
+	    module.run(ctx);
 	    ctx.runUntilDone();
 	    out.freshLine();
 	  }
 	else
 	  {
-	    run();
+            ctx.consumer = VoidConsumer.instance;
+            module.run(ctx);
 	    ctx.runUntilDone();
 	  }
-	// Redundant if registerShutdownHook succeeded (e.g on JDK 1.3).
-	gnu.mapping.OutPort.runCleanups();
+        if (! registered)
+          gnu.mapping.OutPort.runCleanups();
 	exitDecrement();
       }
     catch (Throwable ex)
@@ -271,6 +278,7 @@ public abstract class ModuleBody extends Procedure0
 	//ctx.proc = this; 	ctx.pc = proc.selector;
 	return 0;
       }
+    ctx.where = 0;
     Object[] args = { arg1 };
     return matchN(proc, args, ctx);
   }
@@ -296,6 +304,7 @@ public abstract class ModuleBody extends Procedure0
 	ctx.proc = proc;
 	return 0;
       }
+    ctx.where = 0;
     Object[] args = { arg1, arg2 };
     return matchN(proc, args, ctx);
   }
@@ -324,6 +333,7 @@ public abstract class ModuleBody extends Procedure0
 	// ctx.proc = this; ctx.pc = proc.selector;
 	return 0;
       }
+    ctx.where = 0;
     Object[] args = { arg1, arg2, arg3 };
     return matchN(proc, args, ctx);
   }
@@ -354,6 +364,7 @@ public abstract class ModuleBody extends Procedure0
 	//ctx.proc = this;	ctx.pc = proc.selector;
 	return 0;
       }
+    ctx.where = 0;
     Object[] args = { arg1, arg2, arg3, arg4 };
     return matchN(proc, args, ctx);
   }
