@@ -10,37 +10,49 @@ import java.io.Writer;
 
 /** A primitive Procedure implemented by a plain Java method. */
 
-public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
-{
-  Type retType;
-  /** The types of the method parameters.
-   * If known, the types have been coerced to Language-specific parameters.
-   * Does not include the implicit static link argument of some constructors.
-   */
-  Type[] argTypes;
-  Method method;
-  int op_code;
-  /** 'P' means use invokespecial;
-   * 'V' means expect a target (this) argument, even if method is static;
-   * '\0' means don't expect a target. */
-  char mode;
-  boolean sideEffectFree;
+public class PrimProcedure extends MethodProc implements Inlineable {
+    private Type retType;
 
-  /** If non-null, the LambdaExp that this PrimProcedure implements. */
-  LambdaExp source;
+    /** The types of the method parameters.
+     * If known, the types have been coerced to Language-specific parameters.
+     * Does not include the implicit static link argument of some constructors.
+     */
+    private Type[] argTypes;
 
-  java.lang.reflect.Member member;
+    private Method method;
 
-  public final int opcode() { return op_code; }
+    /** Actual method to invoke. Normally same as method.
+     * However, may have declaring class of the actual caller. 
+     * This is similar to what javac does for improved binary compatibility. */
+    private Method methodForInvoke;
 
-  public Type getReturnType () { return retType; }
-  public void setReturnType (Type retType) { this.retType = retType; }
+    private int op_code;
+    /** 'P' means use invokespecial;
+     * 'V' means expect a target (this) argument, even if method is static;
+     * '\0' means don't expect a target. */
+    private char mode;
+    private boolean sideEffectFree;
 
-  public boolean isSpecial() { return mode == 'P'; }
+    /** If non-null, the LambdaExp that this PrimProcedure implements. */
+    private LambdaExp source;
 
-  public Type getReturnType (Expression[] args) { return retType; }
+    private java.lang.reflect.Member member;
 
-  public Method getMethod () { return method; }
+    public final int opcode() { return op_code; }
+
+    public Type getReturnType () { return retType; }
+    public void setReturnType (Type retType) { this.retType = retType; }
+
+    public boolean isSpecial() { return mode == 'P'; }
+
+    public Type getReturnType (Expression[] args) { return retType; }
+
+    public Method getMethod () { return method; }
+
+    public void setMethodForInvoke(Method m) {
+        methodForInvoke = m;
+        setOpcode(m);
+    }
 
   public boolean isSideEffectFree ()
   {
@@ -93,6 +105,11 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     return app;
   }
 
+    public boolean isAbstract() {
+        return method != null 
+            && (method.getModifiers() & Access.ABSTRACT) != 0;
+    }
+
   public final boolean isConstructor()
   {
     // invokespecial == primitive-constructor
@@ -402,28 +419,31 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
       }
   }
   
+    private void setOpcode(Method m) {
+        int flags = m.getModifiers();
+        if ((flags & Access.STATIC) != 0)
+            this.op_code = 184;  // invokestatic
+        else {
+            ClassType mclass = m.getDeclaringClass();
+            if (mode == 'P')
+                this.op_code = 183;  // invokespecial
+            else {
+                mode = 'V';
+                if ("<init>".equals(m.getName()))
+                    this.op_code = 183;  // invokespecial
+                else if ((mclass.getModifiers() & Access.INTERFACE) != 0)
+                    this.op_code = 185;  // invokeinterface
+                else
+                    this.op_code = 182;  // invokevirtual
+            }
+        }
+    }
+
   private void init(Method method)
   {
     this.method = method;
-    int flags = method.getModifiers();
-    if ((flags & Access.STATIC) != 0)
-      this.op_code = 184;  // invokestatic
-    else
-      {
-	ClassType mclass = method.getDeclaringClass();
-	if (mode == 'P')
-	  this.op_code = 183;  // invokespecial
-        else
-          {
-            mode = 'V';
-            if ("<init>".equals(method.getName()))
-              this.op_code = 183;  // invokespecial
-            else if ((mclass.getModifiers() & Access.INTERFACE) != 0)
-              this.op_code = 185;  // invokeinterface
-            else
-              this.op_code = 182;  // invokevirtual
-          }
-      }
+    this.methodForInvoke = method;
+    setOpcode(method);
     Type[] mtypes = method.getParameterTypes();
     if (isConstructor() && method.getDeclaringClass().hasOuterLink())
       {
@@ -472,6 +492,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     this.op_code = op_code;
     method = classtype.addMethod (name, op_code == 184 ? Access.STATIC : 0,
 				  argTypes, retType);
+    methodForInvoke = method;
     this.retType = retType;
     this.argTypes= argTypes;
     mode = op_code == 184 ? '\0' : 'V';
@@ -516,21 +537,19 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
     // at run-time, in which case we set createVarargsArrayIfNeeded.
     // FIXME This is needless and unreliable complexity.  We should by default
     // create a varargs array - even if the actual argument is an array.
-   boolean createVarargsArrayIfNeeded = false;
+    boolean createVarargsArrayIfNeeded = false;
     if (variable && (method.getModifiers() & Access.VARARGS) != 0
         && nargs > 0 && argTypes.length > 0
         && nargs == arg_count + (is_static ? 0 : 1))
       {
         Type lastType = args[args.length-1].getType();
         Type lastParam = argTypes[argTypes.length-1];
-        if (lastType instanceof ObjectType
-            && lastParam instanceof ArrayType // should always be true
-            && ! (((ArrayType) lastParam).getComponentType()
-                  instanceof ArrayType))
+        if (lastParam.isCompatibleWithValue(lastType) >= 0)
           {
-            if (! (lastType instanceof ArrayType))
-              createVarargsArrayIfNeeded = true;
-            variable = false;
+            if (lastParam instanceof ArrayType // should always be true
+                && (((ArrayType) lastParam).getComponentType()).isCompatibleWithValue(lastType) >= 0)
+                createVarargsArrayIfNeeded = true; 
+             variable = false;
           }
       }
     int fix_arg_count = variable ? arg_count - (is_static ? 1 : 0) : args.length - startArg;
@@ -600,7 +619,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
   public void compile (ApplyExp exp, Compilation comp, Target target)
   {
     gnu.bytecode.CodeAttr code = comp.getCode();
-    ClassType mclass = method == null ? null : method.getDeclaringClass();
+    ClassType mclass = method == null ? null : methodForInvoke.getDeclaringClass();
     Expression[] args = exp.getArgs();
     if (isConstructor())
       {
@@ -697,7 +716,7 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
       }
     else
       {
-        compileInvoke(comp, method, target,
+        compileInvoke(comp, methodForInvoke, target,
                       exp.isTailCall(), op_code, stackType);
       }
   }
@@ -782,9 +801,25 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
                                getDeclaredMethod("getFromContext", 1));
         code.popScope();
       }
-    target.compileFromStack(comp, stackType);
+    if (method.getReturnType() == Type.neverReturnsType)
+      { 
+        // Currently only go here if !takesContext(method).  FIXME: We should
+        // use annotations or something to figure out return type of methods
+        // that take a context (and whose return type is thus void).
+        compileReachedUnexpected(code);
+      }
+    else
+      target.compileFromStack(comp, stackType);
   }
 
+    public static void compileReachedUnexpected(CodeAttr code) {
+        if (code.reachableHere()) {
+            code.emitGetStatic(ClassType.make("gnu.expr.Special")
+                               .getDeclaredField("reachedUnexpected"));
+            code.emitThrow();
+        }
+    }
+
   public Type getParameterType(int index)
   {
     if (takesTarget())
@@ -811,6 +846,72 @@ public class PrimProcedure extends MethodProc implements gnu.expr.Inlineable
   private static ClassLoader systemClassLoader
   = PrimProcedure.class.getClassLoader();
 
+    /** Return the index of the most specific method.
+     * An approximation of the algorithm in JLS3
+     * 15.12.2.5 "Choosing the Most Specific Method." */
+    public static int mostSpecific(PrimProcedure[] procs, int length) {
+        if (length <= 1) // Handles length==0 and length==1.
+            return length - 1;
+        // best is non-negative if there is a single most specific method.
+        int best = 0;
+        // This array (which is allocated lazily) is used if there is is a
+        // set of bestn methods none of which are more specific
+        // than the others.
+        int[] bests = null;
+        // The active length of the bests array.
+        int bestn = 0;
+        outer:
+        for (int i = 1;  i < length;  i++) {
+            PrimProcedure method = procs[i];
+            if (best >= 0) {
+                PrimProcedure winner
+                    = (PrimProcedure) mostSpecific(procs[best], method);
+                if (winner == null) {
+                    if (bests == null)
+                        bests = new int[length];
+                    bests[0] = best;
+                    bests[1] = i;
+                    bestn = 2;
+                    best = -1;
+                } else if (winner == method) {
+                    best = i;
+                    bestn = i;
+                }
+            } else {
+                for (int j = 0;  j < bestn;  j++) {
+                    PrimProcedure old = procs[bests[j]];
+                    PrimProcedure winner
+                        = (PrimProcedure) mostSpecific(old, method);
+                    if (winner == old)
+                        continue outer;
+                    if (winner == null) {
+                        bests[bestn++] = i;
+                        continue outer;
+                    }
+                }
+                // At this point method is more specific than bests[0..bestn-1].
+                best = i;
+                bestn = i;
+            }
+        }
+        if (best < 0 && bestn > 1) {
+            PrimProcedure first = procs[bests[0]];
+            for (int j = 0;  j < bestn;  j++) {
+                int m = bests[j];
+                PrimProcedure method = procs[m];
+                if (j > 0 && ! overrideEquivalent(first, method))
+                    return -1;
+                if (! method.isAbstract()) {
+                    if (best >= 0)
+                        return -1;
+                    best = m;
+                }
+            }
+            return best >= 0 ? best : bests[0];
+        }
+        return best;
+    }
+
   public static PrimProcedure getMethodFor (Procedure pproc, Expression[] args)
   {
     return getMethodFor(pproc, null, args, Language.getDefaultLanguage());
