@@ -55,8 +55,12 @@ public class QuoteExp extends Expression
   static public QuoteExp abstractExp = makeShared(Special.abstractSpecial);
   static public QuoteExp nativeExp = makeShared(Special.nativeSpecial);
   static public QuoteExp voidExp = makeShared(Values.empty, Type.voidType);
-  static public QuoteExp trueExp = makeShared(Boolean.TRUE);
-  static public QuoteExp falseExp = makeShared(Boolean.FALSE);
+    /** Same value as voidExp, but different type, to suppress diagnostics. */
+  static public QuoteExp voidObjectExp = makeShared(Values.empty, Type.objectType);
+    static public QuoteExp trueExp = makeShared(Boolean.TRUE, Type.booleanType);
+  static public QuoteExp falseExp = makeShared(Boolean.FALSE, Type.booleanType);
+    static public QuoteExp trueObjExp = makeShared(Boolean.TRUE);
+  static public QuoteExp falseObjExp = makeShared(Boolean.FALSE);
   static public QuoteExp nullExp = makeShared(null, Type.nullType);
   public static final QuoteExp classObjectExp = makeShared(Type.objectType);
 
@@ -77,7 +81,7 @@ public class QuoteExp extends Expression
       return voidExp;
     // Note we deliberately don't map abstractSpecial to abstractExp.
     if (value instanceof Boolean)
-      return ((Boolean) value).booleanValue() ? trueExp : falseExp;
+      return ((Boolean) value).booleanValue() ? trueObjExp : falseObjExp;
     QuoteExp q = new QuoteExp(value);
     if (position != null)
       q.setLocation(position);
@@ -120,8 +124,9 @@ public class QuoteExp extends Expression
       comp.compileConstant(value, target);
     else
       {
-        comp.compileConstant(value, StackTarget.getInstance(type));
-        target.compileFromStack(comp, type);
+        Type vtype = type.isVoid() ? Type.objectType : type;
+        comp.compileConstant(value, StackTarget.getInstance(vtype));
+        target.compileFromStack(comp, vtype);
       }
   }
  
@@ -162,7 +167,8 @@ public class QuoteExp extends Expression
         if (i == nargs - 1 && ptype != null
             && asMProc.maxArgs() < 0 && i == asMProc.minArgs())
           ptype = null;
-        args[i] = visitor.visit(args[i], ptype);
+        args[i] = visitor.visit(args[i],
+                                InlineCalls.ValueNeededType.make(ptype));
       }
     Compilation comp = visitor.getCompilation();
     if (exp.getFlag(ApplyExp.INLINE_IF_CONSTANT))
