@@ -40,7 +40,7 @@ public abstract class ScopeExp extends Expression
     if (last == null)
       decls = decl;
     else
-      last.next = decl;
+      last.setNext(decl);
     last = decl;
     decl.context = this;
   }
@@ -51,13 +51,13 @@ public abstract class ScopeExp extends Expression
   {
     if (prev == null)
       { // Put first
-        decl.next = decls;
+        decl.setNext(decls);
         decls = decl;
       }
     else
       {
-        decl.next = prev.next;
-        prev.next = decl;
+        decl.setNext(prev.nextDecl());
+        prev.setNext(decl);
       }
     if (last == prev)
       last = decl;
@@ -76,13 +76,13 @@ public abstract class ScopeExp extends Expression
       }
     else
       {
-	oldDecl = prev.next;
-	prev.next = newDecl;
+        oldDecl = prev.nextDecl();
+        prev.setNext(newDecl);
       }
-    newDecl.next = oldDecl.next;
+    newDecl.setNext(oldDecl.nextDecl());
     if (last == oldDecl)
       last = newDecl;
-    oldDecl.next = null;
+    oldDecl.setNext(null);
     newDecl.context = this;
   }
 
@@ -102,12 +102,14 @@ public abstract class ScopeExp extends Expression
 
   public void remove (Declaration prev, Declaration decl)
   {
+    Declaration next = decl.nextDecl();
     if (prev == null)
-      decls = decl.next;
+      decls = next;
     else
-      prev.next = decl.next;
+      prev.setNext(next);
     if (last == decl)
       last = prev;
+    decl.setNext(null);
   }
 
   public ScopeExp () { }
@@ -260,13 +262,10 @@ public abstract class ScopeExp extends Expression
     return n;
   }
 
-  public int countNonDynamicDecls ()
+  public void clearCallList ()
   {
-    int n = 0;
     for (Declaration decl = firstDecl(); decl != null; decl = decl.nextDecl())
-      if (! decl.getFlag(Declaration.IS_DYNAMIC))
-        n++;
-    return n;
+      decl.clearCallList();
   }
 
   public static int nesting (ScopeExp sc)
