@@ -256,7 +256,7 @@ public class SetExp extends AccessExp
             type = decl.getType();
 	    Variable var = decl.getVariable();
 	    if (var == null)
-	      var = decl.allocateVariable(code);
+                var = decl.allocateVariable(code, true);
             int delta = canUseInc(new_value, decl);
             if (delta != BAD_SHORT)
               {
@@ -280,37 +280,34 @@ public class SetExp extends AccessExp
                 code.emitStore(var);
               }
 	  }
-	else if (decl.context instanceof ClassExp && decl.field == null
-		 && ! getFlag(PROCEDURE)
-		 && ((ClassExp) decl.context).isMakingClassPair())
-	  {
-	    String setName = ClassExp.slotToMethodName("set", decl.getName());
-	    ClassExp cl = (ClassExp) decl.context;
-	    Method setter = cl.compiledType.getDeclaredMethod(setName, 1);
-            decl.loadOwningObject(owner, comp);
-	    new_value.compile(comp, decl);
-	    if (needValue)
-	      {
-		code.emitDupX();
-		valuePushed = true;
-	      }
-	    code.emitInvoke(setter);
-	  }
-	else
+        else
 	  {
 	    Field field = decl.field;
-            if (! field.getStaticFlag())
+            boolean isStatic = field != null && field.getStaticFlag();
+            Method setter;
+            if (field == null)
+              {
+                String setName = ClassExp.slotToMethodName("set",
+                                                           decl.getName());
+                ClassExp cl = (ClassExp) decl.context;
+                setter = cl.compiledType.getDeclaredMethod(setName, 1);
+                comp.usedClass(setter.getDeclaringClass());
+              }
+            else
+              {
+                setter = null;
+                comp.usedClass(field.getDeclaringClass());
+              }
+            if (! isStatic)
               decl.loadOwningObject(owner, comp);
-            type = field.getType();
 	    new_value.compile(comp, decl);
             if (! checkReachable(comp))
               return;
-            comp.usedClass(field.getDeclaringClass());
-            if (field.getStaticFlag())
+            if (isStatic)
               {
                 if (needValue)
                   {
-                    code.emitDup(type);
+                    code.emitDup();
                     valuePushed = true;
                   }
                 code.emitPutStatic(field);
@@ -322,7 +319,10 @@ public class SetExp extends AccessExp
                     code.emitDupX();
                     valuePushed = true;
                   }
-                code.emitPutField(field);
+                if (field != null)
+                    code.emitPutField(field);
+                else
+                    code.emitInvoke(setter); 
               }
 	  }
       }
@@ -351,8 +351,8 @@ public class SetExp extends AccessExp
    * Also, if the {@code rhs} is the same as (i.e. a reference to)
    * {@code target}, return zero, to indicate a possible no-op.
    * I.e. we treat {@code target=target} as if it were
-   * {@code target=target+0} - even is {@code tragte} is non-numeric.
-   * @return return increment, or BAD_SHORT.
+   * {@code target=target+0} - even is {@code target} is non-numeric.
+   * @return return increment, or {@code BAD_SHORT}.
    */
   public static int canUseInc (Expression rhs, Declaration target)
   {
