@@ -43,6 +43,14 @@
 		 (try-catch (begin body ...)
 			    (ex <gnu.jemacs.lang.CatchableException>
 				(invoke ex 'match tag))))))
+(define-syntax condition-case
+  (syntax-rules ()
+		((_ var body (handler-name . handler-body) ...)
+                 (try-catch body
+                            (ex gnu.jemacs.lang.CatchableException
+                                (cond ((ex:match 'handler-name) . handler-body)
+                                      ...
+                                      (else (primitive-throw ex))))))))
 
 (define (throw tag value) :: <never-returns>
   (primitive-throw
@@ -62,4 +70,12 @@
 (define (emacs:read #!optional (port (current-input-port)))
   ((<gnu.kawa.lispexpr.LispReader> port):readObject))
 
+(define (princ value #!optional (out (current-output-port))) :: <void>
+  (gnu.jemacs.lang.ELisp:displayFormat:format value out))
 
+(define (prin1 value #!optional (out (current-output-port))) :: <void>
+  (gnu.jemacs.lang.ELisp:writeFormat:format value out))
+
+(define (prefix-numeric-value arg)
+  ;; FIXME
+  arg)
