@@ -55,6 +55,16 @@ public class SwingFrame extends EFrame
     jframe.dispose();
   }
 
+  public void showAboutMessage () 
+  {
+    JOptionPane.showMessageDialog(jframe, aboutMessage());
+  }
+
+  public void showInfoMessage(String msg) 
+  {
+    JOptionPane.showMessageDialog(jframe, msg);
+  }
+
   public String ask(String prompt)
   {
     String result = JOptionPane.showInputDialog(jframe, prompt);
