@@ -28,7 +28,9 @@ public class CompilationHelpers
     if (exp instanceof QuoteExp)
       {
         Object value = ((QuoteExp) exp).getValue();
-        return ! (value instanceof Number || value instanceof Char
+        return ! (value instanceof Number
+                  || value instanceof Boolean
+                  || value instanceof Char
                   || value instanceof Symbol);
       }
     return false;
@@ -185,7 +187,11 @@ public class CompilationHelpers
     if ((nonNumeric(args[0]) || nonNumeric(args[1]))
         && ! maybeLazy(args[0]) && ! maybeLazy(args[1]))
       return new ApplyExp(((IsEqv) proc).isEq, args);
-    return exp;
+    Method meth =
+        ClassType.make("gnu.kawa.functions.IsEqv")
+        .getDeclaredMethod("apply", 2);
+    return new ApplyExp(new PrimProcedure(meth, visitor.getLanguage()),
+                        args).setLine(exp);
   }
 }
 
