@@ -102,6 +102,8 @@ public class CompileMisc implements Inlineable
             args[1] = visitor.visit(args[1], type);
             CompileReflect.checkKnownClass(type, comp);
             exp.setType(type);
+            if (args[1].getType() == type)
+                return args[1];
             return exp;
           }
       }
@@ -138,6 +140,7 @@ public class CompileMisc implements Inlineable
             xargs[0] = new QuoteExp(Integer.valueOf(0), Type.intType);
             System.arraycopy(args, skip, xargs, 1, xargs.length-1);
             ApplyExp ae = new ApplyExp(typeFormat.getDeclaredMethod("formatToString", 2), xargs);
+            ae.setLine(exp);
             ae.setType(Type.javalangStringType);
             return ae;
           }
@@ -152,6 +155,7 @@ public class CompileMisc implements Inlineable
                 args = xargs;
               }
             ApplyExp ae = new ApplyExp(typeFormat.getDeclaredMethod("formatToWriter", 3), args);
+            ae.setLine(exp);
             ae.setType(Type.voidType);
             return ae;
           }
@@ -263,7 +267,12 @@ public class CompileMisc implements Inlineable
       throw new Error ("wrong number of arguments to "+proc.getName());
     CodeAttr code = comp.getCode();
     Type type = Scheme.getTypeValue(args[0]);
-    if (type != null)
+    if (type == Type.neverReturnsType)
+      {
+        args[1].compile(comp, Target.Ignore);
+        PrimProcedure.compileReachedUnexpected(code);
+      }
+    else if (type != null)
       {
         args[1].compile(comp, Target.pushValue(type));
 	if (code.reachableHere())
@@ -360,113 +369,133 @@ public class CompileMisc implements Inlineable
       }
   }
 
-  public static void compileNumberCompare (NumberCompare proc, ApplyExp exp, Compilation comp, Target target)
-  {
-    Expression[] args = exp.getArgs();
-    if (args.length == 2)
-      {
-	Expression arg0 = args[0];
-	Expression arg1 = args[1];
-	int kind0 = classifyForNumCompare(arg0);
-	int kind1 = classifyForNumCompare(arg1);
-	CodeAttr code = comp.getCode();
-	if (kind0 > 0 && kind1 > 0
-            && kind0 <= Arithmetic.REALNUM_CODE && kind1 <= Arithmetic.REALNUM_CODE
-	    // Don't optimize if both operands are fractions. FIXME???
-	    && (kind0 != Arithmetic.RATNUM_CODE || kind1 != Arithmetic.RATNUM_CODE))
-	  {
-	    if (! (target instanceof ConditionalTarget))
-	      {
-		IfExp.compile(exp, QuoteExp.trueExp, QuoteExp.falseExp,
-			      comp, target);
-		return;
-	      }
-	    int mask = proc.flags;
-	    if (mask == NumberCompare.TRUE_IF_NEQ)
-	      mask = NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
-	    if (kind0 <= Arithmetic.INTNUM_CODE && kind1 <= Arithmetic.INTNUM_CODE
-		&& (kind0 > Arithmetic.LONG_CODE || kind1 > Arithmetic.LONG_CODE))
-	      {
-		Type[] ctypes = new Type[2];
-		ctypes[0] = Arithmetic.typeIntNum;
-		if (kind1 <= Arithmetic.LONG_CODE)
-		  {
-		    ctypes[1] = Type.longType;
-		  }
-		else if (kind0 <= Arithmetic.LONG_CODE
-			 // Simple check to avoid re-ordering side-effects.
-			 && (arg0 instanceof QuoteExp
-			     || arg1 instanceof QuoteExp
-			     || arg0 instanceof ReferenceExp
-			     || arg1 instanceof ReferenceExp))
-		  {
-		    ctypes[1] = Type.longType;
-		    args = new Expression[2];
-		    args[0] = arg1;
-		    args[1] = arg0;
-		    if (mask != NumberCompare.TRUE_IF_EQU && mask != NumberCompare.TRUE_IF_GRT+NumberCompare.TRUE_IF_LSS)
-		      mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
-		  }
-		else
-		  ctypes[1] = Arithmetic.typeIntNum;
-		Method cmeth
-		  = Arithmetic.typeIntNum.getMethod("compare", ctypes);
-		PrimProcedure compare = new PrimProcedure(cmeth);
-		arg0 = new ApplyExp(compare, args);
-		arg1 = new QuoteExp(IntNum.zero());
-		kind0 = kind1 = Arithmetic.INT_CODE;
-	      }
-	    Type commonType;
-	    if (kind0 <= Arithmetic.INT_CODE && kind1 <= Arithmetic.INT_CODE)
-	      commonType = Type.intType;
-	    else if (kind0 <= Arithmetic.LONG_CODE && kind1 <= Arithmetic.LONG_CODE)
-	      commonType = Type.longType;
-	    else
-	      commonType = Type.doubleType;
-	    StackTarget subTarget = new StackTarget(commonType);
-	    ConditionalTarget ctarget = (ConditionalTarget) target;
+    public static void compileNumberCompare(NumberCompare proc, ApplyExp exp, Compilation comp, Target target) {
+        CodeAttr code = comp.getCode();
+        Expression[] args = exp.getArgs();
+        if (args.length == 2) {
+            Expression arg0 = args[0];
+            Expression arg1 = args[1];
+            int kind0 = classifyForNumCompare(arg0);
+            int kind1 = classifyForNumCompare(arg1);
+            if (kind0 > 0 && kind1 > 0
+                && kind0 <= Arithmetic.REALNUM_CODE && kind1 <= Arithmetic.REALNUM_CODE
+                // Don't optimize if both operands are fractions. FIXME???
+                && (kind0 != Arithmetic.RATNUM_CODE || kind1 != Arithmetic.RATNUM_CODE)) {
+                if (! (target instanceof ConditionalTarget)) {
+                    IfExp.compile(exp, QuoteExp.trueExp, QuoteExp.falseExp,
+                                  comp, target);
+                    return;
+                }
+                int mask = proc.flags;
+                if (mask == NumberCompare.TRUE_IF_NEQ)
+                    mask = NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
+                if (kind0 <= Arithmetic.INTNUM_CODE
+                    && kind1 <= Arithmetic.INTNUM_CODE
+                    && (kind0 > Arithmetic.LONG_CODE
+                        || kind1 > Arithmetic.LONG_CODE)) {
+                    Type[] ctypes = new Type[2];
+                    ctypes[0] = Arithmetic.typeIntNum;
+                    if (kind1 <= Arithmetic.LONG_CODE) {
+                        ctypes[1] = Type.longType;
+                    } else if (kind0 <= Arithmetic.LONG_CODE
+                               // Simple check to avoid re-ordering side-effects.
+                               && (arg0 instanceof QuoteExp
+                                   || arg1 instanceof QuoteExp
+                                   || arg0 instanceof ReferenceExp
+                                   || arg1 instanceof ReferenceExp)) {
+                        ctypes[1] = Type.longType;
+                        args = new Expression[2];
+                        args[0] = arg1;
+                        args[1] = arg0;
+                        if (mask != NumberCompare.TRUE_IF_EQU && mask != NumberCompare.TRUE_IF_GRT+NumberCompare.TRUE_IF_LSS)
+                            mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
+                    } else
+                        ctypes[1] = Arithmetic.typeIntNum;
+                    Method cmeth
+                        = Arithmetic.typeIntNum.getMethod("compare", ctypes);
+                    PrimProcedure compare = new PrimProcedure(cmeth);
+                    arg0 = new ApplyExp(compare, args);
+                    arg1 = new QuoteExp(IntNum.zero());
+                    kind0 = kind1 = Arithmetic.INT_CODE;
+                }
+                Type commonType;
+                if (kind0 <= Arithmetic.INT_CODE && kind1 <= Arithmetic.INT_CODE)
+                    commonType = Type.intType;
+                else if (kind0 <= Arithmetic.LONG_CODE && kind1 <= Arithmetic.LONG_CODE)
+                    commonType = Type.longType;
+                else
+                    commonType = Type.doubleType;
+                StackTarget subTarget = new StackTarget(commonType);
+                ConditionalTarget ctarget = (ConditionalTarget) target;
 	    
-	    int opcode;
-	    if (arg0 instanceof QuoteExp && ! (arg1 instanceof QuoteExp))
-	      {
-		Expression tmp = arg1; arg1 = arg0; arg0 = tmp;
-		if (mask != NumberCompare.TRUE_IF_EQU && mask !=NumberCompare. TRUE_IF_GRT+NumberCompare.TRUE_IF_LSS)
-		  mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
-	      }
-	    Label label1 = ctarget.trueBranchComesFirst ? ctarget.ifFalse : ctarget.ifTrue;
-	    if (ctarget.trueBranchComesFirst)
-	      mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU;
-	    switch (mask)
-	      {
-	      case NumberCompare.TRUE_IF_GRT:  opcode = 157 /*ifgt*/;  break;
-	      case NumberCompare.TRUE_IF_EQU:  opcode = 153 /*ifeq*/;  break;
-	      case NumberCompare.TRUE_IF_LSS:  opcode = 155 /*iflt*/;  break;
-	      case NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS:  opcode = 154 /*ifne*/;  break;
-	      case NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU:  opcode = 156 /*ifge*/;  break;
-	      case NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU:  opcode = 158 /*ifle*/;  break;
-	      default:
-		opcode = 0;
-	      }
-	    arg0.compile(comp, subTarget);
-	    Object value;
-	    if (kind0 <= Arithmetic.INT_CODE && kind1 <= Arithmetic.INT_CODE
-		&& arg1 instanceof QuoteExp
-		&& (value = ((QuoteExp) arg1).getValue()) instanceof IntNum
-		&& ((IntNum) value).isZero())
-	      {
-		code.emitGotoIfCompare1(label1, opcode);
-	      }
-	    else
-	      {
-		arg1.compile(comp, subTarget);
-		code.emitGotoIfCompare2(label1, opcode);
-	      }
-	    ctarget.emitGotoFirstBranch(code);
-	    return;
-	  }
-      }
-    ApplyExp.compile(exp, comp, target);
-  }
+                int opcode;
+                if (arg0 instanceof QuoteExp && ! (arg1 instanceof QuoteExp)) {
+                    Expression tmp = arg1; arg1 = arg0; arg0 = tmp;
+                    if (mask != NumberCompare.TRUE_IF_EQU && mask !=NumberCompare. TRUE_IF_GRT+NumberCompare.TRUE_IF_LSS)
+                        mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS;
+                }
+                Label label1 = ctarget.trueBranchComesFirst ? ctarget.ifFalse : ctarget.ifTrue;
+                if (ctarget.trueBranchComesFirst)
+                    mask ^= NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU;
+                switch (mask) {
+                case NumberCompare.TRUE_IF_GRT:
+                    opcode = 157 /*ifgt*/;  break;
+                case NumberCompare.TRUE_IF_EQU:
+                    opcode = 153 /*ifeq*/;  break;
+                case NumberCompare.TRUE_IF_LSS:
+                    opcode = 155 /*iflt*/;  break;
+                case NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_LSS:
+                    opcode = 154 /*ifne*/;  break;
+                case NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU:
+                    opcode = 156 /*ifge*/;  break;
+                case NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU:
+                    opcode = 158 /*ifle*/;  break;
+                default:
+                    opcode = 0;
+                }
+                arg0.compile(comp, subTarget);
+                Object value;
+                if (kind0 <= Arithmetic.INT_CODE && kind1 <= Arithmetic.INT_CODE
+                    && arg1 instanceof QuoteExp
+                    && (value = ((QuoteExp) arg1).getValue()) instanceof IntNum
+                    && ((IntNum) value).isZero()) {
+                    code.emitGotoIfCompare1(label1, opcode);
+                } else {
+                    arg1.compile(comp, subTarget);
+                    code.emitGotoIfCompare2(label1, opcode);
+                }
+                ctarget.emitGotoFirstBranch(code);
+                return;
+            }
+        }
+
+        String mname;
+        switch (proc.flags) {
+        case NumberCompare.TRUE_IF_GRT:
+            mname = "$Gr";  break;
+        case NumberCompare.TRUE_IF_EQU:
+            mname = "$Eq";  break;
+        case NumberCompare.TRUE_IF_LSS:
+            mname = "$Ls";  break;
+        case NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU:
+            mname = "$Gr$Eq";  break;
+        case NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU:
+            mname = "$Ls$Eq";  break;
+        default:
+            mname = null;
+        }
+        if (mname != null) {
+            ClassType compclass =
+                ClassType.make("gnu.kawa.functions.NumberCompare");
+            Method meth = args.length == 2
+                ? compclass.getDeclaredMethod(mname, 2)
+                : compclass.getDeclaredMethod(mname+"$V", 4);
+            new ApplyExp(meth, args).setLine(exp)
+                .compile(comp, target);
+            return;
+        }
+        ApplyExp.compile(exp, comp, target);
+    }
 
   static int classifyForNumCompare (Expression exp)
   {
