@@ -14,6 +14,7 @@ import gnu.kawa.xml.KNode;
 import gnu.xml.XMLPrinter;
 /* #endif */
 import gnu.kawa.xml.XmlNamespace;
+import gnu.kawa.lispexpr.LispLanguage;
 import gnu.text.PrettyWriter;
 import gnu.text.Printable;
 /* #ifdef use:java.util.regex */
@@ -33,8 +34,23 @@ public class DisplayFormat extends AbstractFormat
   public static final ThreadLocation outRadix
     = new ThreadLocation("out-radix");
 
-  /** This field is set true by Scheme when it creates a shared write format */
-  public boolean checkSharing = false;
+    public static final DisplayFormat schemeDisplayFormat
+        = new DisplayFormat(false, 'S');
+
+    public static final DisplayFormat schemeWriteSimpleFormat
+        = new DisplayFormat(true, 'S');
+    public static final DisplayFormat schemeWriteFormat
+        = new DisplayFormat(true, 'S');
+    public static final DisplayFormat schemeWriteSharedFormat
+        = new DisplayFormat(true, 'S');
+    static {
+        schemeWriteFormat.checkSharing = 0;
+        schemeWriteSharedFormat.checkSharing = 1;
+    }
+
+    /** Controls whether we check for sharing and cycles.
+     * 1: check for sharing; 0: check for cycles: -1: no checking. */
+    public int checkSharing = -1;
 
   /** Create a new instance.
    * @param readable if output should be formatted so it could be read
@@ -122,6 +138,7 @@ public class DisplayFormat extends AbstractFormat
     PrettyWriter pout = out.getPrettyWriter();
     // The stack of position markers, populated by CDR'ing further into the list.
     int[] posnStack = null;
+    Object[] tailStack = null;
     int stackTail = 0;
     Object list = value;
     out.startLogicalBlock("(", false, ")");
@@ -142,7 +159,7 @@ public class DisplayFormat extends AbstractFormat
 	    writeObject(LList.checkNonList(list), (Consumer) out);
 	    break;
 	  }
-	if (checkSharing)
+	if (checkSharing >= 0)
 	  {
 	    
 	    int hashIndex = pout.IDHashLookup(list);
@@ -152,15 +169,21 @@ public class DisplayFormat extends AbstractFormat
 	      {
 		// writePositionMarker will return the index to which is was enqueued
 		posn = pout.writePositionMarker(true);
-		if (posnStack == null) 
+		if (posnStack == null) {
 		  posnStack = new int[128]; // should be plently for most cases.
+                  tailStack = new Object[128];
+                }
 		else if (stackTail >= posnStack.length)
 		  {
-		    int[] newStack = new int[posnStack.length << 1];
-		    System.arraycopy(posnStack, 0, newStack, 0, stackTail);
-		    posnStack = newStack;
+		    int[] newPStack = new int[posnStack.length << 1];
+		    System.arraycopy(posnStack, 0, newPStack, 0, stackTail);
+		    posnStack = newPStack;
+                    Object[] newTStack = new Object[posnStack.length << 1];
+		    System.arraycopy(tailStack, 0, newTStack, 0, stackTail);
+		    tailStack = newTStack;
 		  }
-		posnStack[stackTail++] = posn;
+		posnStack[stackTail] = posn;
+                tailStack[stackTail++] = list;
 		// Mark (hash) this object
 		pout.IDHashPutAtIndex(list, posn, hashIndex);
 	      }
@@ -174,8 +197,11 @@ public class DisplayFormat extends AbstractFormat
 	      }
 	  }
       }
-    for (;--stackTail >= 0;)
+    for (;--stackTail >= 0;) {
       pout.writePairEnd(posnStack[stackTail]);
+      if (checkSharing == 0)
+          pout.IDHashRemove(tailStack[stackTail]);
+    }
 
     out.endLogicalBlock(")");
   }
@@ -200,7 +226,8 @@ public class DisplayFormat extends AbstractFormat
         ((OutPort) out).writeWordStart();
         space = true;
       }
-    if (checkSharing && isInteresting(obj))
+    boolean removeNeeded = false;
+    if (checkSharing >= 0 && isInteresting(obj))
       {
 	// The value returned from this hash is the respective index in the
 	// queueInts[] from PrettyWriter to which this object should reference.
@@ -210,9 +237,10 @@ public class DisplayFormat extends AbstractFormat
 	  {
 	    // Find the position in the queueInts that future (if any) backreferences
 	    // should reference
-	    posn =  pout.writePositionMarker(false);
+	    int nposn = pout.writePositionMarker(false);
 	    // Mark (hash) this object
-	    pout.IDHashPutAtIndex(obj, posn, hashIndex);
+	    pout.IDHashPutAtIndex(obj, nposn, hashIndex);
+            removeNeeded = checkSharing == 0;
 	    // Print the object, instead of emitting print-circle notation
 	    skip = false;
 	  }
@@ -230,6 +258,8 @@ public class DisplayFormat extends AbstractFormat
       }
     if (!skip)
       writeObjectRaw(obj, out);
+    if (removeNeeded)
+        pout.IDHashRemove(obj);
     if (space)
       ((OutPort) out).writeWordEnd();
   }
@@ -249,11 +279,19 @@ public class DisplayFormat extends AbstractFormat
     else if (obj instanceof Symbol)
       {
         Symbol sym = (Symbol) obj;
-        if (sym.getNamespace() == XmlNamespace.HTML)
+        Namespace ns = sym.getNamespace();
+        if (ns == XmlNamespace.HTML)
           {
             write("html:", out);
             write(sym.getLocalPart(), out);
           }
+        else if (ns == LispLanguage.entityNamespace
+                 || ns == LispLanguage.constructNamespace)
+          {
+            write(ns.getPrefix(), out);
+            write(":", out);
+            write(sym.getLocalPart(), out);
+          }
         else
           writeSymbol(sym, out, readable);
       }
@@ -355,7 +393,13 @@ public class DisplayFormat extends AbstractFormat
       ((Consumable) obj).consume(out);
     else if (obj instanceof Printable)
       ((Printable) obj).print(out);
-    else if (obj instanceof RatNum)
+    else if (obj instanceof RatNum
+             || (obj instanceof Number
+                 && (obj instanceof Long
+                     || obj instanceof Integer
+                     || obj instanceof Short
+                     || obj instanceof Byte
+                     || obj instanceof java.math.BigInteger)))
       {
         int b = 10;
         boolean showRadix = false;
@@ -369,7 +413,7 @@ public class DisplayFormat extends AbstractFormat
           b = ((IntNum) base).intValue();
         else if (base != null)
           b = Integer.parseInt(base.toString());
-        String asString = ((RatNum) obj).toString(b);
+        String asString = Arithmetic.asRatNum(obj).toString(b);
         if (showRadix)
           {
             if (b == 16)
@@ -518,44 +562,76 @@ public class DisplayFormat extends AbstractFormat
       out.write(':');
   }
 
-  void writeSymbol (String sym, Consumer out, boolean readable)
-  {
-    /* #ifdef use:java.util.regex */
-    /* Use |...| if symbol doesn't follow R5RS conventions
-       for identifiers or has a colon in the interior. */
-    if (readable && ! r5rsIdentifierMinusInteriorColons.matcher(sym).matches())
-      {
-        int len = sym.length();
-        if (len == 0)
-          {
-            write("||", out);
-          }
-        else
-          {
-            boolean inVerticalBars = false;
-            for (int i = 0;  i < len;  i++)
-              {
-                char ch = sym.charAt(i);
-                if (ch == '|')
-                  {
-                    write(inVerticalBars ? "|\\" : "\\", out);
-                    inVerticalBars = false;
-                  }
-                else if (! inVerticalBars)
-                  {
+    void writeSymbol (String sym, Consumer out, boolean readable) {
+        /* #ifdef use:java.util.regex */
+        /* Use |...| if symbol doesn't follow R5RS conventions
+           for identifiers or has a colon in the interior. */
+        if (readable
+            && ! r5rsIdentifierMinusInteriorColons.matcher(sym).matches()) {
+            int len = sym.length();
+            boolean r7rsStyle = true; // FIXME
+            if (r7rsStyle) {
+                out.write('|');
+                for (int i = 0;  i < len;  i++) {
+                    int ch = sym.charAt(i);
+                    if (ch >= 0xD800 && ch <= 0xDBFF) { // surrogates
+                        char next = sym.charAt(++i);
+                        if (next >= 0xDC00 && next <= 0xDFFF)
+                           ch = ((ch - 0xD800) << 10)
+                               + (next - 0xDC00) + 0x10000;
+                    }
+                    if (ch == '|' || ch == '\\' || ch < ' ' || ch == 127) {
+                        out.write('\\');
+                        switch(ch) {
+                        case '\007': out.write('a'); break;
+                        case '\b': out.write('b'); break;
+                        case '\n': out.write('n'); break;
+                        case '\r': out.write('r'); break;
+                        case '\t': out.write('t'); break;
+                        case '\\': out.write('\\'); break;
+                        case '|': out.write('|'); break;
+                        default:
+                            out.write('x');
+                            writeHexDigits(ch, out);
+                            out.write(';');
+                        }
+                    }
+                    else
+                        Char.print(ch, out);
+                }
+                out.write('|');
+            } else if (len == 0) {
+                write("||", out);
+            } else {
+                boolean inVerticalBars = false;
+                for (int i = 0;  i < len;  i++) {
+                    char ch = sym.charAt(i);
+                    if (ch == '|') {
+                        write(inVerticalBars ? "|\\" : "\\", out);
+                        inVerticalBars = false;
+                    } else if (! inVerticalBars) {
+                        out.write('|');
+                        inVerticalBars = true;
+                    }
+                    out.write(ch);
+                }
+                if (inVerticalBars)
                     out.write('|');
-                    inVerticalBars = true;
-                  }
-                out.write(ch);
-              }
-            if (inVerticalBars)
-              out.write('|');
-          }
-        return;
-      }
-    /* #endif */
-    write(sym, out);
-  }
+            }
+            return;
+        }
+        /* #endif */
+        write(sym, out);
+    }
+
+    static void writeHexDigits(int i, Consumer out) {
+        int high = i >>> 4;
+        if (high != 0) {	  
+            writeHexDigits(high, out);
+            i &= 15;
+	}
+        out.write("0123456789ABCDEF".charAt(i));
+    }
   
   /**
    * An "interesting" object is one where object identity is significant.
