@@ -139,13 +139,7 @@ public class LangObjType extends ObjectType implements TypeValue
                     REGEX_TYPE_CODE);
 
   public static final LangObjType stringType =
-    new LangObjType("string",
-                    /* #ifdef use:java.lang.CharSequence */
-                    "java.lang.CharSequence",
-                    /* #else */
-                    // /* better would be a union of CharSeq and j.l.String. */
-                    // "gnu.lists.CharSeq",
-                    /* #endif */
+    new LangObjType("string", "java.lang.CharSequence",
                     STRING_TYPE_CODE);
 
   public static final LangObjType listType =
@@ -203,27 +197,6 @@ public class LangObjType extends ObjectType implements TypeValue
         if (other == typeClassType)
           return 1;
         break;
-      case INTEGER_TYPE_CODE:
-        if (other instanceof PrimType)
-          {
-            char sig1 = other.getSignature().charAt(0);
-            switch (sig1)
-              {
-              case 'I': case 'J':  case 'S':  case 'B':
-                return 1;
-              }
-          }
-      case DFLONUM_TYPE_CODE:
-      case REAL_TYPE_CODE:
-        if (other instanceof PrimType)
-          {
-            char sig1 = other.getSignature().charAt(0);
-            switch (sig1)
-              {
-              case 'D': case 'F':
-                return 1;
-              }
-          }
       }
     return getImplementationType().compare(other.getImplementationType());
   }
@@ -755,6 +728,8 @@ public class LangObjType extends ObjectType implements TypeValue
         return new PrimProcedure("gnu.text.URIPath", "makeURI", 1);
       case VECTOR_TYPE_CODE:
         return new PrimProcedure("gnu.lists.FVector", "make", 1);
+      case U8VECTOR_TYPE_CODE:
+        return new PrimProcedure("kawa.lib.bytevectors", "$make$bytevector$"+VARARGS_SUFFIX, 1);
       case LIST_TYPE_CODE:
         return gnu.kawa.functions.MakeList.list;
       case STRING_TYPE_CODE:
