@@ -17,10 +17,10 @@ import kawa.lang.Syntax; // FIXME
 
 public abstract class LispLanguage extends Language
 {
-  static public final String quote_sym = "quote";
-  static public final String unquote_sym = "unquote";
-  static public final String unquotesplicing_sym = "unquote-splicing";
-  static public final String quasiquote_sym = "quasiquote";
+  static public final String quote_str = "quote";
+  static public final String unquote_str = "unquote";
+  static public final String unquotesplicing_str = "unquote-splicing";
+  static public final String quasiquote_str = "quasiquote";
   /** Used for Kawa infix ':' operator. */
   static public final Symbol lookup_sym = Namespace.EmptyNamespace.getSymbol("$lookup$");
   // FUTURE: Used for: [ e1 e2 ... ]
@@ -34,6 +34,19 @@ public abstract class LispLanguage extends Language
   public static StaticFieldLocation getNamedPartLocation =
     new StaticFieldLocation("gnu.kawa.functions.GetNamedPart", "getNamedPart");
   static { getNamedPartLocation.setProcedure(); }
+  
+  /**
+   * The unit namespace contains the bindings for symbols such as `cm',
+   * `s', etc.
+   */
+  public static final Namespace unitNamespace =
+      Namespace.valueOf("http://kawa.gnu.org/unit", "unit");
+
+  public static final Namespace constructNamespace =
+      Namespace.valueOf("http://kawa.gnu.org/construct", "$construct$");
+
+  public static final Namespace entityNamespace =
+      Namespace.valueOf("http://kawa.gnu.org/entity", "$entity$");
 
   /** The default <code>ReadTable</code> for this language. */
   protected ReadTable defaultReadTable;
@@ -202,6 +215,7 @@ public abstract class LispLanguage extends Language
             types.put("rational", LangObjType.rationalType);
             types.put("integer", LangObjType.integerType);
             types.put("symbol", ClassType.make("gnu.mapping.Symbol"));
+            types.put("simple-symbol", ClassType.make("gnu.mapping.SimpleSymbol"));
             types.put("namespace", ClassType.make("gnu.mapping.Namespace"));
             types.put("keyword", ClassType.make("gnu.expr.Keyword"));
             types.put("pair", ClassType.make("gnu.lists.Pair"));
