@@ -120,57 +120,60 @@ public class LispReader extends Lexer
   }
 
     public Object readValues (int ch,  ReadTable rtable, int sharingIndex)
-      throws java.io.IOException, SyntaxException
-  {
-    return readValues(ch, rtable.lookup(ch), rtable, sharingIndex);
-  }
+            throws java.io.IOException, SyntaxException {
+        return readValues(ch, rtable.lookup(ch), rtable, sharingIndex);
+    }
 
-  /** May return zero or multiple values.
-   * Returns no values if looking at whitespace or a comment. */
-  public Object readValues (int ch, ReadTableEntry entry, ReadTable rtable, int sharingIndex)
-      throws java.io.IOException, SyntaxException
-  {
-    // Step numbers refer to steps in section 2.2 of the HyperSpec.
-    // Step 1:
-    int startPos = tokenBufferLength;
+    /** May return zero or multiple values.
+     * Returns no values if looking at whitespace or a comment. */
+    public Object readValues (int ch, ReadTableEntry entry, ReadTable rtable,
+                              int sharingIndex)
+        throws java.io.IOException, SyntaxException {
+        seenEscapes = false;
+        return entry.read(this, ch, -1, sharingIndex);
+    }
 
-    seenEscapes = false;
-    int kind = entry.getKind();
-    Object result;
-    switch (kind)
-      {
-      case ReadTable.ILLEGAL:
-	// Step 2:
-	String err = ("invalid character #\\"+((char) ch));  // FIXME
-	if (interactive) fatal(err);
-	else error(err);
-	return Values.empty;
-      case ReadTable.WHITESPACE:
-	// Step 3:
-	return Values.empty;
-      case ReadTable.TERMINATING_MACRO:
-      case ReadTable.NON_TERMINATING_MACRO:
-	  return entry.read(this, ch, -1, sharingIndex);
-      case ReadTable.CONSTITUENT:
-      case ReadTable.SINGLE_ESCAPE: // Step 5:
-      case ReadTable.MULTIPLE_ESCAPE: // Step 6:
-      default:  // 
-	  result = readAndHandleToken(ch, startPos, rtable);
-      }
-    return bindSharedObject(sharingIndex, result);
-  }
+    public Pair readValuesAndAppend(int ch, ReadTable rtable, Pair last)
+            throws java.io.IOException, SyntaxException {
+        int line = port.getLineNumber();
+        int column = port.getColumnNumber();
+        Object values = readValues(ch, rtable, -1);
+        int index = 0;
+        int next = Values.nextIndex(values, index);
+        if (next >= 0) {
+            for (;;) {
+                Object value = Values.nextValue(values, index);
+                index = next;
+                if (value == gnu.expr.QuoteExp.voidExp)
+                    value = Values.empty;
+                next = Values.nextIndex(values, index);
+                if (next < 0)
+                    value = handlePostfix(value, rtable, line, column);
+                Pair pair = makePair(value, line, column);
+                setCdr(last, pair);
+                last = pair;
+                if (next < 0)
+                    break;
+            }
+        }
+        return last;
+    }
 
   protected Object readAndHandleToken(int ch, int startPos, ReadTable rtable)
     throws java.io.IOException, SyntaxException
   {
-    readToken(ch, getReadCase(), rtable);
+    char readCase = getReadCase();
+    readToken(ch, rtable);
     int endPos = tokenBufferLength;
     if (! seenEscapes)
       {
         Object value = parseNumber(tokenBuffer, startPos, endPos - startPos,
                                    '\0', 0, SCM_NUMBERS);
         if (value != null && ! (value instanceof String))
-          return value;
+          {
+            tokenBufferLength = startPos;
+            return value;
+          }
         /* Common Lisp only?  FIXME
         if (isPotentialNumber(tokenBuffer, startPos, endPos))
           {
@@ -181,7 +184,6 @@ public class LispReader extends Lexer
         */
       }
 
-    char readCase = getReadCase();
     if (readCase == 'I')
       {
 	int upperCount = 0;
@@ -260,6 +262,7 @@ public class LispReader extends Lexer
 
     int len = endPos - startPos;
 
+    Object result;
     if (lbrace >= 0 && rbrace > lbrace)
       {
         String prefix = lbrace > 0 ? new String(tokenBuffer, startPos, lbrace-startPos) : null;
@@ -271,22 +274,24 @@ public class LispReader extends Lexer
         if (! (rightOperand instanceof SimpleSymbol))
           error("expected identifier in symbol after '{URI}:'");
         // FIXME should allow "compound keyword" - for attribute names
-        return Symbol.valueOf(rightOperand.toString(), uri, prefix);
+        result = Symbol.valueOf(rightOperand.toString(), uri, prefix);
       }
-
-    if (rtable.initialColonIsKeyword && packageMarker == startPos && len > 1)
+    else if (rtable.initialColonIsKeyword && packageMarker == startPos && len > 1)
       {
 	startPos++;
 	String str = new String(tokenBuffer, startPos, endPos-startPos);
-	return Keyword.make(str.intern());
+	result = Keyword.make(str.intern());
     }
-    if (rtable.finalColonIsKeyword && packageMarker == endPos - 1
+    else if (rtable.finalColonIsKeyword && packageMarker != -1 && packageMarker == endPos - 1
         && (len > 1 || seenEscapes))
       {
 	String str = new String(tokenBuffer, startPos, len - 1);
-	return Keyword.make(str.intern());
+	result = Keyword.make(str.intern());
       }
-    return rtable.makeSymbol(new String(tokenBuffer, startPos, len));
+    else
+      result = rtable.makeSymbol(new String(tokenBuffer, startPos, len));
+    tokenBufferLength = startPos;
+    return result;
   }
 
   public static final char TOKEN_ESCAPE_CHAR = '\uffff';
@@ -296,7 +301,10 @@ public class LispReader extends Lexer
    */
   protected boolean seenEscapes;
 
-  void readToken(int ch, char readCase, ReadTable rtable)
+    /** Read token, leaving characters in tokenBuffer.
+     * Sets seenEscapes if escape characters are seen.
+     */
+  void readToken(int ch, ReadTable rtable)
       throws java.io.IOException, SyntaxException
   {
     boolean inEscapes = false;
@@ -346,10 +354,15 @@ public class LispReader extends Lexer
 	    if (ch < 0)
 	      eofError("unexpected EOF after single escape");
             if (rtable.hexEscapeAfterBackslash
-                && (ch == 'x' || ch == 'X'))
-              ch = readHexEscape();
-	    tokenBufferAppend(TOKEN_ESCAPE_CHAR);
-	    tokenBufferAppend(ch);
+                // We've allowed hex escapes for a while.
+                // Allow R7RS general escapes - but only inside |bars|.
+                && (inEscapes || ch == 'x' || ch == 'X'))
+                ch = readEscape(ch);
+            if (ch >= 0)
+              {
+                tokenBufferAppend(TOKEN_ESCAPE_CHAR);
+                tokenBufferAppend(ch);
+              }
 	    seenEscapes = true;
 	    continue;
 	  }
@@ -373,7 +386,7 @@ public class LispReader extends Lexer
 	      case ReadTable.CONSTITUENT:
                 if (ch == '{' && entry == ReadTableEntry.brace)
                   braceNesting++;
-                /* ... fall thotugh ... */
+                /* ... fall through ... */
 	      case ReadTable.NON_TERMINATING_MACRO:
 		tokenBufferAppend(ch);
 		continue;
@@ -393,6 +406,18 @@ public class LispReader extends Lexer
       }
   }
 
+    public String readTokenString(int ch, ReadTable rtable)
+            throws java.io.IOException, SyntaxException {
+        int startPos = tokenBufferLength;
+        if (ch >= 0)
+            tokenBufferAppend(ch);
+        readToken(read(), rtable);
+        int length = tokenBufferLength - startPos;
+        String str = new String(tokenBuffer, startPos, length);
+        tokenBufferLength = startPos;
+        return str;
+    }
+
     public Object readObject() throws java.io.IOException, SyntaxException {
 	return readObject(-1, false);
     }
@@ -433,59 +458,55 @@ public class LispReader extends Lexer
       }
   }
 
-  protected boolean validPostfixLookupStart (int ch, ReadTable rtable)
-      throws java.io.IOException
-  {
-    if (ch < 0 || ch == rtable.postfixLookupOperator)
-      return false;
-    if (ch == ',')
-      return true;
-    int kind = rtable.lookup(ch).getKind();
-    return kind == ReadTable.CONSTITUENT
-      || kind == ReadTable.NON_TERMINATING_MACRO
-      || kind == ReadTable.MULTIPLE_ESCAPE
-      || kind == ReadTable.SINGLE_ESCAPE;
-  }
+    protected boolean validPostfixLookupStart (int ch, ReadTable rtable)
+            throws java.io.IOException {
+        if (ch < 0 || ch == rtable.postfixLookupOperator)
+            return false;
+        if (ch == ',')
+            return true;
+        int kind = rtable.lookup(ch).getKind();
+        return kind == ReadTable.CONSTITUENT
+            || kind == ReadTable.NON_TERMINATING_MACRO
+            || kind == ReadTable.MULTIPLE_ESCAPE
+            || kind == ReadTable.SINGLE_ESCAPE;
+    }
 
-  Object handlePostfix (Object value, ReadTable rtable, int line, int column)
-      throws java.io.IOException, SyntaxException
-  {
-    if (value == QuoteExp.voidExp)
-      value = Values.empty;
-    for (;;)
-      {
-        int ch = port.peek();
-        if (ch == '[' && rtable.defaultBracketMode == -2)
-          {
-            port.read();
-            Object lst = ReaderParens.readList(this, null, ch, 1, ']', -1);
-            value = PairWithPosition.make(value, lst,
-                                          port.getName(), line+1, column+1);
-            value = PairWithPosition.make(LispLanguage.bracket_apply_sym, value,
-                                          port.getName(), line+1, column+1);
-          }
-        else if (ch == rtable.postfixLookupOperator)
-          {
-            // A kludge to map PreOpWord to ($lookup$ Pre 'Word).
-            port.read();
-            int ch2 = port.peek();
-            if (! validPostfixLookupStart(ch2, rtable))
-              {
-                unread();
+    /** After reading a value check for following {@code '['} or {@code ':'}.
+     */
+    Object handlePostfix (Object value, ReadTable rtable, int line, int column)
+        throws java.io.IOException, SyntaxException {
+        if (value == QuoteExp.voidExp)
+            value = Values.empty;
+        for (;;) {
+            int ch = port.peek();
+            String str; int slen;
+            if (ch == '[' && rtable.defaultBracketMode == -2) {
+                port.read();
+                Object lst = ReaderParens.readList(this, null, ch, 1, ']', -1);
+                value = PairWithPosition.make(value, lst,
+                                              port.getName(), line+1, column+1);
+                value = PairWithPosition.make(LispLanguage.bracket_apply_sym, value,
+                                              port.getName(), line+1, column+1);
+            } else if (ch == rtable.postfixLookupOperator) {
+                // A kludge to map PreOpWord to ($lookup$ Pre 'Word).
+                port.read();
+                int ch2 = port.peek();
+                if (! validPostfixLookupStart(ch2, rtable)) {
+                    unread();
+                    break;
+                }
+                ch = port.read();
+                Object rightOperand = readValues(ch, rtable.lookup(ch), rtable, -1);
+                value = LList.list2(value,
+                                    LList.list2(rtable.makeSymbol(LispLanguage.quote_str), rightOperand));
+                value = PairWithPosition.make(LispLanguage.lookup_sym, value,
+                                              port.getName(), line+1, column+1);
+            }
+            else
                 break;
-              }
-            ch = port.read();
-            Object rightOperand = readValues(ch, rtable.lookup(ch), rtable, -1);
-            value = LList.list2(value,
-                                LList.list2(rtable.makeSymbol(LispLanguage.quasiquote_sym), rightOperand));
-            value = PairWithPosition.make(LispLanguage.lookup_sym, value,
-                                          port.getName(), line+1, column+1);
-          }
-        else
-          break;
-      }
-    return value;
-  }
+        }
+        return value;
+    }
 
   private boolean isPotentialNumber (char[] buffer, int start, int end)
   {
@@ -775,15 +796,17 @@ public class LispReader extends Lexer
         if (sign_seen
             && pos + 4 < end && buffer[pos+3] == '.' && buffer[pos+4] == '0')
           {
-            if (buffer[pos] == 'i'
-                && buffer[pos+1] == 'n'
-                && buffer[pos+2] == 'f')
+            char b0 = buffer[pos];
+            char b1, b2;
+            if ((b0 == 'i' || b0 == 'I')
+                && ((b1 = buffer[pos+1]) == 'n' || b1 == 'N')
+                && ((b2 = buffer[pos+2]) == 'f' || b2 == 'F'))
               {
                 infnan = 'i';
               }
-            else if (buffer[pos] == 'n'
-                && buffer[pos+1] == 'a'
-                && buffer[pos+2] == 'n')
+            else if ((b0 == 'n' || b0 == 'N')
+                     && ((b1 = buffer[pos+1]) == 'a' || b1 == 'A')
+                     && ((b2 = buffer[pos+2]) == 'n' || b2 == 'N'))
               {
                 infnan = 'n';
               }
@@ -987,6 +1010,7 @@ public class LispReader extends Lexer
       case 'r':  c = 13;  break;  // carriage return
       case 'e':  c = 27;  break;  // escape
       case '\"': c = 34;  break;  // quote
+      case '|':  c = '|';  break;  // vertical bar
       case '\\': c = 92;  break;  // backslash
       case ' ':  // Skip to end of line, inclusive.
       case '\n': // Skip initial whitespace on following line.
@@ -1163,6 +1187,11 @@ public class LispReader extends Lexer
       return Pair.make(car, cdr);
   }
 
+    protected Pair makePair2 (Object car, Object cadr, Object cddr,
+                              int line, int column) {
+        return makePair(car, makePair(cadr, cddr, line, column), line, column);
+    }
+
   protected void setCar (Object pair, Object car)
   {
     ((Pair) pair).setCarBackdoor(car);
@@ -1182,7 +1211,7 @@ public class LispReader extends Lexer
     throws java.io.IOException, SyntaxException
   {
     int startPos = reader.tokenBufferLength - previous;
-    reader.readToken(reader.read(), 'P', ReadTable.getCurrent());
+    reader.readToken(reader.read(), ReadTable.getCurrent());
     int endPos = reader.tokenBufferLength;
     if (startPos == endPos)
       {
@@ -1213,7 +1242,7 @@ public class LispReader extends Lexer
       reader.eofError("unexpected EOF in character literal");
     int startPos = reader.tokenBufferLength;
     reader.tokenBufferAppend(ch);
-    reader.readToken(reader.read(), 'D', ReadTable.getCurrent());
+    reader.readToken(reader.read(), ReadTable.getCurrent());
     char[] tokenBuffer = reader.tokenBuffer;
     int length = reader.tokenBufferLength - startPos;
     if (length == 1)
@@ -1264,19 +1293,38 @@ public class LispReader extends Lexer
       reader.eofError("unexpected EOF in #! special form");
 
     /* Handle Unix #!PROGRAM line at start of file. */
-    if (ch == '/'
+    if ((ch == '/' || ch == ' ')
 	&& reader.getLineNumber() == 0
 	&& reader.getColumnNumber() == 3)
       {
-	ReaderIgnoreRestOfLine.getInstance().read(reader, '#', 1);
-	return Values.empty;
+        String filename = reader.getName();
+        if (filename != null
+            && ApplicationMainSupport.commandName.get(null) == null)
+          {
+            ApplicationMainSupport.commandName.set(filename);
+          }
+
+        boolean sawBackslash = false;
+        for (;;)
+          {
+            ch = reader.read();
+            if (ch < 0)
+              break;
+            if (ch == '\\')
+              sawBackslash = true;
+            else if (ch == '\n' || ch == '\r')
+              {
+                if (! sawBackslash)
+                  break;
+                sawBackslash = false;
+              }
+            else if (sawBackslash && ch != ' ' && ch != '\t')
+              sawBackslash = false;
+          }
+        return Values.empty;
       }
 
-    int startPos = reader.tokenBufferLength;
-    reader.tokenBufferAppend(ch);
-    reader.readToken(reader.read(), 'D', ReadTable.getCurrent());
-    int length = reader.tokenBufferLength - startPos;
-    String name = new String(reader.tokenBuffer, startPos, length);
+    String name = reader.readTokenString(ch, ReadTable.getCurrent());
     if (name.equals("optional"))
       return Special.optional;
     if (name.equals("rest"))
@@ -1328,6 +1376,13 @@ public class LispReader extends Lexer
 	  break;
 	size = size * 10 + digit;
       }
+    return readSimpleVector(reader, kind, ch, size);
+  }
+
+  public static SimpleVector
+      readSimpleVector(LispReader reader, char kind, int ch, int size)
+    throws java.io.IOException, SyntaxException
+  {
     if (! (size == 8 || size == 16 || size == 32 || size == 64)
         || (kind == 'F' && size < 32)
         || ch != '(')
@@ -1370,4 +1425,6 @@ public class LispReader extends Lexer
       }
     return null;
   }
+
+    boolean deprecatedXmlEnlosedReported;
 }
