@@ -36,10 +36,6 @@ public class ReadTable extends RangeTable
    */
   protected boolean hexEscapeAfterBackslash = true;
 
-    public static ReaderQuote unquoteEntry =
-        new ReaderQuote(makeSymbol(LispLanguage.unquote_sym),
-                        '@', makeSymbol(LispLanguage.unquotesplicing_sym));
-
   /** Set whether ":IDENTIFIER" should be treated as a keyword. */
   public void setInitialColonIsKeyword (boolean whenInitial)
   {
@@ -58,7 +54,7 @@ public class ReadTable extends RangeTable
   {
   }
 
-  public void initialize ()
+  public void initialize(boolean sharpIsTerminating)
   {
     ReadTableEntry entry;
     entry = ReadTableEntry.getWhitespaceInstance();
@@ -77,7 +73,7 @@ public class ReadTable extends RangeTable
     set('!',  entry);
     set('$',  entry);
     set('%',  entry);
-    set('&',  entry);
+    set('&',  ReadTableEntry.ampersand);
     set('*',  entry);
     set('+',  entry);
     set('-',  entry);
@@ -95,12 +91,15 @@ public class ReadTable extends RangeTable
     set('\b', entry);
     set(':',  new ReaderColon());
     set('\"', new ReaderString());
-    set('#',  ReaderDispatch.create(this));
+    set('#',  ReaderDispatch.create(this, ! sharpIsTerminating));
     set(';',  ReaderIgnoreRestOfLine.getInstance());
     set('(',  ReaderParens.getInstance('(', ')'));
 
-    set('\'', new ReaderQuote(makeSymbol(LispLanguage.quote_sym)));
-    set('`',  new ReaderQuote(makeSymbol(LispLanguage.quasiquote_sym)));
+    set('\'', new ReaderQuote(makeSymbol(LispLanguage.quote_str)));
+    set('`',  new ReaderQuote(makeSymbol(LispLanguage.quasiquote_str)));
+    ReaderQuote unquoteEntry =
+        new ReaderQuote(makeSymbol(LispLanguage.unquote_str),
+                        '@', makeSymbol(LispLanguage.unquotesplicing_str));
     set(',',  unquoteEntry);
 
     setBracketMode();  // Sets the entries for '[', ']', and '<'.
@@ -110,7 +109,7 @@ public class ReadTable extends RangeTable
   public static ReadTable createInitial ()
   {
     ReadTable tab = new ReadTable();
-    tab.initialize();
+    tab.initialize(true);
     return tab;
   }
 
@@ -151,7 +150,7 @@ public class ReadTable extends RangeTable
   }
 
   /** Specify how '[' and ']' are handled.
-   * Overless overridden, uses defaultBracketMode. */
+   * Unless overridden, uses defaultBracketMode. */
   public void setBracketMode()
   {
     setBracketMode(defaultBracketMode);
@@ -249,7 +248,7 @@ public class ReadTable extends RangeTable
     return entry;
   }
 
-  static Object makeSymbol (String name)
+  protected Object makeSymbol (String name)
   {
     return Symbol.valueOf(name);
   }
