@@ -48,23 +48,30 @@ public class ReaderDispatchMisc extends ReadTableEntry
       case ':':
 	// Handle Guile-style keyword syntax: '#:KEYWORD'
 	// Note this conflicts with Common Lisp uninterned symbols.  FIXME
-	int startPos = reader.tokenBufferLength;
-	reader.readToken(reader.read(), 'P', ReadTable.getCurrent());
-	length = reader.tokenBufferLength - startPos;
-	name = new String(reader.tokenBuffer, startPos, length);
-	reader.tokenBufferLength = startPos;
-	return gnu.expr.Keyword.make(name.intern());
+        name = reader.readTokenString(-1, ReadTable.getCurrent());
+        return gnu.expr.Keyword.make(name.intern());
       case '\\':
 	return LispReader.readCharacter(reader);
       case '!':
 	return LispReader.readSpecial(reader);
       case 'T':
-	return Boolean.TRUE;
       case 'F':
-	ch = in.peek();
-	if (Character.isDigit((char) ch))
-	  return LispReader.readSimpleVector(reader, 'F');
-	return Boolean.FALSE;
+          name = reader.readTokenString(ch, ReadTable.getCurrent());
+          String nameLC = name.toLowerCase();
+          if (nameLC.equals("t") || nameLC.equals("true"))
+              return Boolean.TRUE;
+          if (nameLC.equals("f") || nameLC.equals("false"))
+              return Boolean.FALSE;
+          int size;
+          if (nameLC.equals("f32")) size = 32;
+          else if (nameLC.equals("f64")) size = 64;
+          else
+            {
+              in.error("unexpected characters following '#'");
+              return Boolean.FALSE;
+            }
+          return LispReader.readSimpleVector(reader, 'F',
+                                             reader.read(), size);
       case 'S':
       case 'U':
 	return LispReader.readSimpleVector(reader, (char) ch);
@@ -93,21 +100,7 @@ public class ReaderDispatchMisc extends ReadTableEntry
       return readRegex(in, ch, count);
       /* #endif */
       case '|':
-	port = reader.getPort();
-	if (port instanceof InPort)
-	  {
-	    saveReadState = ((InPort) port).readState;
-	    ((InPort) port).readState = '|';
-	  }
-	try
-	  {
-	    reader.readNestedComment('#', '|');
-	  }
-	finally
-	  {
-	    if (port instanceof InPort)
-	      ((InPort) port).readState = saveReadState;
-	  }
+        readNestedComment(reader);
 	return Values.empty;
       case ';':
 	port = reader.getPort();
@@ -151,6 +144,22 @@ public class ReaderDispatchMisc extends ReadTableEntry
       }
   }
 
+    public static void readNestedComment(LispReader reader)
+            throws java.io.IOException, SyntaxException {
+	LineBufferedReader port = reader.getPort();
+        char saveReadState = '\0';
+	if (port instanceof InPort) {
+	    saveReadState = ((InPort) port).readState;
+	    ((InPort) port).readState = '|';
+        }
+	try {
+	    reader.readNestedComment('#', '|');
+        } finally {
+	    if (port instanceof InPort)
+                ((InPort) port).readState = saveReadState;
+        }
+    }
+
   /* #ifdef use:java.util.regex */
   public static Pattern readRegex (Lexer in, int ch, int count)
     throws java.io.IOException, SyntaxException
