@@ -1446,7 +1446,8 @@ that want to provide \"user-friendly,\" wide-spectrum interfaces to
 their clients."
   (cond ((string? x) (string->char-set x))
         ((character? x) (char-set x))
-        ((char-set? x) x)))
+        ((char-set? x) x)
+        (else (primitive-throw (java.lang.ClassCastException "not converible to char-set")))))
 
 (define (char-set-size (cs ::char-set)) ::int
   "Returns the number of elements in character set CS."
@@ -1723,7 +1724,8 @@ allowed, but not required, to side-effect its first argument."
         ((null? l2) l1)
         ((or (> (cdar l2) (cdar l1))        ; swap arguments if
              (and (= (cdar l2) (cdar l1))   ; necessary to ensure
-                  (< (caar l2) (cdar l1)))) ; (cdar l1) is highest
+                  (< (caar l2) (cdar l1))   ; (cdar l1) is highest
+                  (> (caar l2) (caar l1))))
          (%boundary-pairs-union l2 l1))
         (else
          (let ((ending ::int (cdar l1)))
