@@ -90,11 +90,7 @@
     (<gnu.expr.SetExp> 
      `(set ,(! get-symbol exp) ,(unrewrite (! get-new-value exp))))
     (<gnu.expr.LambdaExp>
-     `(lambda ,(packing (pack)
-                 (do ((decl :: <gnu.expr.Declaration>
-			    (! first-decl exp) (! next-decl decl)))
-                     ((eq? decl #!null))
-                   (pack (! get-symbol decl))))
+     `(lambda ,(unrewrite-arglist exp)
 	,(unrewrite (@ body exp))))
     (<gnu.expr.ReferenceExp> (! get-symbol exp))
     (<gnu.expr.ApplyExp> (unrewrite-apply exp))
@@ -107,6 +103,41 @@
 		  (list (unrewrite eclause))))))
     (#t exp)))
 
+(define (unrewrite-arglist (exp <gnu.expr.LambdaExp>))
+  (let* ((min (@ min_args exp))
+	 (rest? (negative? (@ max_args exp)))
+	 (key? (not (eq? (@ keywords exp) #!null)))
+	 (opt (@ opt_args exp))
+	 (required '())
+	 (optional '())
+	 (key '())
+	 (rest #f)
+	 (aux '()))
+    (do ((decl :: <gnu.expr.Declaration> (! first-decl exp)
+	       (! next-decl decl))
+	 (i 0 (+ i 1)))
+	((eq? decl #!null))
+      (let ((var (! get-symbol decl)))
+	(cond ((< i min) 
+	       (set! required (cons var required)))
+	      ((< i (+ min opt))
+	       (set! optional (cons var optional)))
+	      ((and rest? (= i (+ min opt)))
+	       (set! rest var))
+	      ((and key? (< i (+ min opt 
+				 (if rest? 1 0)
+				 (@ length (@ keywords exp)))))
+	       (set! key (cons var key)))
+	      (#t 
+	       (error "nyi")))))
+    `(,@(reverse required)
+      ,@(cond ((zero? opt) ())
+	      (#t `(#!optional . ,(reverse optional))))
+      ,@(cond (rest? `(#!rest ,rest))
+	      (#t '()))
+      ,@(cond (key? `(#!key . ,(reverse key)))
+	      (#t '())))))
+
 (define (unrewrite* (exps <gnu.expr.Expression[]>))
   (packing (pack)
     (do ((len (@ length exps))
